/* File: gosiv_types.h

* Copyright (c) 2003 James A. Strother
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*/

#ifndef _GOSIV_TYPES_H
#define _GOSIV_TYPES_H 1

#include <assert.h>
#include <string>
using namespace std;

enum
  {
    GOSIV_BACK_SOLID,
    GOSIV_BACK_NONE
  };

enum
  {
    GOSIV_ARROWHEAD_OUTLINE,
    GOSIV_ARROWHEAD_LINES,
    GOSIV_ARROWHEAD_CURVY,
    GOSIV_ARROWHEAD_NONE
  };

enum
  {
    GOSIV_ARROWTAIL_LINE,
    GOSIV_ARROWTAIL_BLOCK,
    GOSIV_ARROWTAIL_NONE
  };

enum
  {
    GOSIV_LEGEND_SOLID,
    GOSIV_LEGEND_NONE
  };

enum
  {
    GOSIV_LEGEND_TOP_LEFT,
    GOSIV_LEGEND_TOP_RIGHT,
    GOSIV_LEGEND_BOTTOM_LEFT,
    GOSIV_LEGEND_BOTTOM_RIGHT
  };

enum
  {
    GOSIV_FONT_COURIER,
    GOSIV_FONT_TIMES,
    GOSIV_FONT_TIMES_BOLD,
    GOSIV_FONT_TIMES_ITALIC,
    GOSIV_FONT_HELVETICA,
    GOSIV_FONT_HELVETICA_BOLD,
    GOSIV_FONT_HELVETICA_ITALIC
  };

enum
  {
    GOSIV_JUSTIFY_LEFT,
    GOSIV_JUSTIFY_CENTER,
    GOSIV_JUSTIFY_RIGHT
  };

enum
  {
    GOSIV_JUSTIFY_TOP,
    GOSIV_JUSTIFY_MIDDLE,
    GOSIV_JUSTIFY_BOTTOM
  };

struct gosiv_point
{
  double x;
  double y;
};

struct gosiv_color
{
  double  r;
  double  g;
  double  b;

  gosiv_color();
  gosiv_color(double,double,double);
};

struct gosiv_image
{
  int      nrows;
  int      ncols;
  double*  array;

  gosiv_image();
};

struct gosiv_canvas
{
  double  scale;

  double  xlower;
  double  xupper;
  double  ylower;
  double  yupper;

  double  width;
  double  height;

  gosiv_canvas();
};

struct gosiv_back
{
  int          type;
  double       stroke_weight;
  gosiv_color  stroke_color;
  gosiv_color  fill_color;
  
  gosiv_back();
};

struct gosiv_arrow
{
  double       x;
  double       y;
  double       u;
  double       v;

  double       scale;

  int          tail_type;
  double       tail_width;
  double       tail_stroke_weight;
  gosiv_color  tail_stroke_color;
  gosiv_color  tail_fill_color;

  int          head_type;
  double       head_width;
  double       head_height;
  double       head_angle;
  double       head_stroke_weight;
  gosiv_color  head_stroke_color;
  gosiv_color  head_fill_color;

  gosiv_arrow();
};

struct gosiv_legend
{
  int          type;
  int          placement;
  double       horiz_margin;
  double       vert_margin;
  double       horiz_inset;
  double       vert_inset;
  double       stroke_weight;
  gosiv_color  stroke_color;
  gosiv_color  fill_color;

  double       arrow_length;
  string       arrow_label;
  int          arrow_label_font;
  gosiv_color  arrow_label_color;
  double       arrow_label_size;
  gosiv_arrow  arrow;
  
  double       ruler_length;
  double       ruler_width;
  double       ruler_stroke_weight;
  gosiv_color  ruler_stroke_color;
  gosiv_color  ruler_fill_color;
  string       ruler_label;
  int          ruler_label_font;
  gosiv_color  ruler_label_color;
  double       ruler_label_size;

  gosiv_legend();
};

#endif
