/* File: alg1.c 
  
* Copyright (c) 2002 James A. Strother 
* 
* This program is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2 of the License, or 
* (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public License 
* along with this program; if not, write to the Free Software 
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
* 
*/ 
 
#include "osiv.h" 
 
void OsivAlg1_CheckOpts(osiv_input* input) 
{ 
  if (input->PreProc == OSIV_IOINT_BLANK) OsivError("preprocessing method undefined"); 
  if (input->Interp == OSIV_IOINT_BLANK) OsivError("interpolation method undefined"); 
  if (input->XSize == OSIV_IOINT_BLANK) OsivError("image width undefined"); 
  if (input->YSize == OSIV_IOINT_BLANK) OsivError("image height undefined"); 
  if (input->XStart == OSIV_IOINT_BLANK) OsivError("x start position undefined"); 
  if (input->YStart == OSIV_IOINT_BLANK) OsivError("y start position undefined"); 
  if (input->XOffset == OSIV_IOINT_BLANK) OsivError("x offset undefined"); 
  if (input->YOffset == OSIV_IOINT_BLANK) OsivError("y offset undefined"); 
  if (input->XVectSpace == OSIV_IOINT_BLANK) OsivError("x spacing undefined"); 
  if (input->YVectSpace == OSIV_IOINT_BLANK) OsivError("y spacing undefined"); 
  if (input->XNumVects == OSIV_IOINT_BLANK) OsivError("number of x vectors undefined"); 
  if (input->YNumVects == OSIV_IOINT_BLANK) OsivError("number of y vectors undefined"); 
  if (input->XWinSize == OSIV_IOINT_BLANK) OsivError("x window size undefined"); 
  if (input->YWinSize == OSIV_IOINT_BLANK) OsivError("y window size undefined"); 
  if (input->XMaxDisp == OSIV_IOINT_BLANK) OsivError("maximum x displacement undefined"); 
  if (input->YMaxDisp == OSIV_IOINT_BLANK) OsivError("maximum y displacement undefined"); 
  if (input->FrmStart == OSIV_IOINT_BLANK) OsivError("start frame undefined"); 
  if (input->FrmFinish == OSIV_IOINT_BLANK) OsivError("stop frame undefined"); 
  if (input->FrmSkip == OSIV_IOINT_BLANK) OsivError("frame jump undefined"); 
  if (input->FrmSet == OSIV_IOINT_BLANK) OsivError("frame spacing undefined"); 
  if (!strlen(input->MovieFN)) OsivError("movie file undefined"); 
  if (!strlen(input->VectorsFN)) OsivError("vector file undefined"); 
 
  if (input->PreProc < 0 || input->PreProc > 5) OsivError("unrecognized preprocessing method"); 
  if (input->Interp < 0 || input->Interp > 4) OsivError("unrecognized interpolation method"); 
  if (input->XSize < 1) OsivError("image width must be non-zero, non-negative"); 
  if (input->YSize < 1) OsivError("image height must be non-zero, non-negative"); 
  if (input->XStart < 0) OsivError("x start position must be non-negative"); 
  if (input->YStart < 0) OsivError("y start position must be non-negative"); 
  if (input->XVectSpace < 1) OsivError("x spacing must be non-zero, non-negative"); 
  if (input->YVectSpace < 1) OsivError("y spacing must be non-zero, non-negative"); 
  if (input->XNumVects < 1) OsivError("number of x vectors must be non-zero, non-negative"); 
  if (input->YNumVects < 1) OsivError("number of y vectors must be non-zero, non-negative"); 
  if (input->XWinSize < 1) OsivError("window x size must be non-zero, non-negative"); 
  if (input->YWinSize < 1) OsivError("window y size must be non-zero, non-negative"); 
  if (input->XMaxDisp < 1) OsivError("maximum x displacement must be non-zero, non-negative"); 
  if (input->YMaxDisp < 1) OsivError("maximum y displacement must be non-zero, non-negative"); 
  if (input->FrmStart < 0) OsivError("start frame must be non-negative"); 
  if (input->FrmFinish < 0) OsivError("stop frame must be non-negative"); 
  if (input->FrmSkip < 1) OsivError("frame jump must be non-zero, non-negative"); 
  if (input->FrmSet < 1) OsivError("frame spacing must be non-zero, non-negative"); 
 
  if ((log(2*input->XMaxDisp+1)+log(2*input->YMaxDisp+1))/log(2) >= OSIV_DISP_BITS) 
    OsivError("max displacement exceeds that specified at compile time"); 
  if ((log(input->XWinSize)+log(input->YWinSize))/log(2) >= OSIV_WINDOW_BITS) 
    OsivError("window size exceeds that specified at compile time"); 
  if ((log(input->XSize)+log(input->YSize))/log(2) >= OSIV_FRAME_BITS) 
    OsivError("image size exceeds that specified at compile time"); 
  if (log(input->FrmFinish)/log(2) >= OSIV_MOVIE_BITS) 
    OsivError("movie length exceeds that specified at compile time"); 
 
  if (input->XStart + input->XOffset < input->XMaxDisp) 
    OsivError("x start position is too small"); 
  if (input->YStart + input->YOffset < input->YMaxDisp) 
    OsivError("y start position is too small"); 
  if (input->XSize < input->XStart + (input->XNumVects-1)* 
      input->XVectSpace+input->XWinSize) 
    OsivError("x interrogation domain exceeds limits of image"); 
  if (input->YSize < input->YStart + (input->YNumVects-1)* 
      input->YVectSpace+input->YWinSize) 
    OsivError("y interrogation domain exceeds limits of image"); 
  if (input->XSize < input->XStart + input->XOffset + input->XMaxDisp +  
      (input->XNumVects-1)*input->XVectSpace+input->XWinSize) 
    OsivError("x interrogation domain exceeds limits of image"); 
  if (input->YSize < input->YStart + input->YOffset + input->YMaxDisp +  
      (input->YNumVects-1)*input->YVectSpace+input->YWinSize) 
    OsivError("y interrogation domain exceeds limits of image"); 
  if (input->FrmFinish < input->FrmStart) 
    OsivError("last frame must follow first");   
} 
 
static void OsivAlg1_SumWindow (osiv_param* p, 
				osiv_imcp* im0, 
				osiv_imcp* im1, 
				osiv_immw* map) 
{ 
  register osiv_intf i,j; 
  register osiv_immw sum; 
   
  sum = 0; 
  for (i = 0; i < p->YWinSize*p->XSize; i+=p->XSize) 
    { 
      for (j = i; j < p->XWinSize+i; j++) 
	{ 
	  osiv_imsp d = *(im0+j)-*(im1+j); 
	  sum += d*d; 
	} 
    } 
  *map = sum; 
} 
 
static void OsivAlg1_GetErrorMap (osiv_param* p, 
				  osiv_imcp* im0, 
				  osiv_imcp* im1, 
				  osiv_immw* map,  
				  osiv_intd* minInd) 
{ 
  osiv_intf i,j; 
  osiv_intd k; 
  
  *minInd = k = 0; 
  for (i = -p->YMaxDisp; i <= p->YMaxDisp; i++) 
    { 
      for (j = -p->XMaxDisp+i*p->XSize; 
	   j <= p->XMaxDisp+i*p->XSize; j++) 
	{ 
	  OsivAlg1_SumWindow(p,im0+j,im1,map+k); 
 
	  if (*(map+k) <= *(map+*minInd)) 
	    *minInd = k; 
	  k++; 
	} 
    } 
} 
 
static void OsivAlg1_ConvertMap (osiv_param* p, 
				 osiv_ioreal* dest, 
				 osiv_immw* src) 
{ 
  osiv_intw i; 
  for (i = 0; i < p->MapLength; i++) 
    *(dest+i) = (osiv_ioreal)*(src+i); 
} 
 
static void OsivAlg1_ConvertVects (osiv_param* p, 
				   osiv_ioreal* dest, 
				   osiv_real* src) 
{ 
  if (p->SwapReal) 
    { 
      osiv_intf i; 
      for (i = 0; i < 2*p->NumVects; i++) 
	*(dest+i) = (osiv_ioreal)*(src+i); 
    } 
} 
 
void OsivAlg1 (osiv_param *p) 
{ 
  osiv_intm i; 
  osiv_imcp* im0; 
  osiv_imcp* im1; 
  osiv_immw* map; 
  osiv_real* vects; 
  osiv_ioreal* mapStd; 
  osiv_ioreal* vectsStd; 
 
  /* Allocate memory */ 
  im0 = (osiv_imcp*)OsivMalloc(p->ImageLength*sizeof(osiv_imcp)); 
  im1 = (osiv_imcp*)OsivMalloc(p->ImageLength*sizeof(osiv_imcp)); 
  map = (osiv_immw*)OsivMalloc(p->MapLength*sizeof(osiv_immw)); 
  vects = (osiv_real*)OsivMalloc(2*p->NumVects*sizeof(osiv_real)); 
 
  if (!p->WriteMap) 
    mapStd = NULL; 
  else 
    mapStd = (osiv_ioreal*)OsivMalloc(p->MapLength*sizeof(osiv_ioreal)); 
 
  if (!p->SwapReal) 
    vectsStd = (osiv_ioreal*)vects; 
  else 
    vectsStd = (osiv_ioreal*)OsivMalloc(2*p->NumVects*sizeof(osiv_ioreal)); 
 
  /* Open body tag */ 
  OsivWriteTag(p->VectorsFile,OSIV_TAG_BODY); 
 
  /* Run through images */ 
  for (i = p->FrmStart; i <= p->FrmFinish; i += p->FrmSkip) 
    { 
      osiv_intf j,k; 
      osiv_real* vects_ptr = vects; 
      osiv_imcp* im0Start,*im1Start; 
       
      /* Read the ith image */ 
      if (p->FrmSkip != p->FrmSet || 
	  i == p->FrmStart) 
	{ 
	  OsivGetImage(p,i,im0); 
	  OsivPreProc(p,i,im0); 
	} 
      else 
	{ 
	  osiv_imcp* tmp = im1; 
	  im1 = im0; im0 = tmp; 
	} 
 
      /* Read the ith image pair */ 
      OsivGetImage(p,i+p->FrmSet,im1); 
      OsivPreProc(p,i+p->FrmSet,im1); 
 
      /* Open frame tag */ 
      OsivWriteTag(p->VectorsFile,OSIV_TAG_FRAME); 
 
      /* Write frame number */ 
      OsivWriteFrameNum(p->VectorsFile,i); 
 
      /* Set start locations */ 
      im0Start = im0 + p->XStart+p->XOffset  
	+(p->YStart+p->YOffset)*p->XSize; 
      im1Start = im1 + p->XStart+p->YStart*p->XSize; 
 
      /* Run through each of the windows */ 
      for (j = 0; j < p->YNumVects; j++) 
	{ 
	  for (k = 0; k < p->XNumVects; k++) 
	    { 
	      osiv_intf pos; 
	      osiv_intd ind,ix,iy; 
 
	      /* Move to new window */ 
	      pos = k*p->XVectSpace 
		+ j*p->YVectSpace*p->XSize; 
 
	      /* Calculate maps */ 
	      OsivAlg1_GetErrorMap(p,im0Start+pos,im1Start+pos,map,&ind); 
	      ix = ind%p->XMapSize;   iy = ind/p->XMapSize; 
 
	      /* Calculate actual displacement */ 
	      *(vects_ptr) = ix + p->XOffset - p->XMaxDisp; 
	      *(vects_ptr+1) = iy + p->YOffset - p->YMaxDisp; 
 
	      /* Perform interpolation */ 
	      if (ix > 0 && ix < p->XMapSize-1 && 
		  iy > 0 && iy < p->YMapSize-1) 
		OsivInterpMin_IMMW(p->Interp,map+ind, 
				   -1,1, 
				   -p->XMapSize,p->XMapSize, 
				   vects_ptr,vects_ptr+1); 
 
	      /* Write error map to file */ 
	      if (p->WriteMap) 
		{ 
		  OsivAlg1_ConvertMap(p,mapStd,map); 
		  OsivWriteTag(p->VectorsFile,OSIV_TAG_BOX); 
		  OsivWriteBoxNum(p->VectorsFile,k+j*p->XNumVects); 
		  OsivWriteMap(p->VectorsFile,p->MapLength,mapStd); 
		  OsivWriteTag(p->VectorsFile,OSIV_GAT_BOX); 
		} 
 
	      vects_ptr += 2; 
	    } 
	} 
 
      /* Write vectors to file */ 
      OsivAlg1_ConvertVects(p,vectsStd,vects); 
      OsivWriteVectors(p->VectorsFile,2*p->NumVects,vectsStd); 
       
      /* Close frame tag */ 
      OsivWriteTag(p->VectorsFile,OSIV_GAT_FRAME); 
       
    } 
 
  /* Close body tag */ 
  OsivWriteTag(p->VectorsFile,OSIV_GAT_BODY); 
 
  /* Free memory */ 
  free(im0); 
  free(im1); 
  free(map); 
  free(vects); 
 
  if (p->WriteMap) 
    free(mapStd); 
  if (p->SwapReal)  
    free(vectsStd); 
} 
  
   
