/* File: alg2.c 
 
* Copyright (c) 2002 James A. Strother 
* 
* This program is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2 of the License, or 
* (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public License 
* along with this program; if not, write to the Free Software 
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
* 
*/ 
 
#include "osiv.h" 
 
void OsivAlg2_CheckOpts (osiv_input* input) 
{ 
  if (input->PreProc == OSIV_IOINT_BLANK) OsivError("preprocessing method undefined"); 
  if (input->Interp == OSIV_IOINT_BLANK) OsivError("interpolation method undefined"); 
  if (input->XSize == OSIV_IOINT_BLANK) OsivError("image width undefined"); 
  if (input->YSize == OSIV_IOINT_BLANK) OsivError("image height undefined"); 
  if (input->XStart == OSIV_IOINT_BLANK) OsivError("x start position undefined"); 
  if (input->YStart == OSIV_IOINT_BLANK) OsivError("y start position undefined"); 
  if (input->XOffset == OSIV_IOINT_BLANK) OsivError("x offset undefined"); 
  if (input->YOffset == OSIV_IOINT_BLANK) OsivError("y offset undefined"); 
  if (input->XVectSpace == OSIV_IOINT_BLANK) OsivError("x spacing undefined"); 
  if (input->YVectSpace == OSIV_IOINT_BLANK) OsivError("y spacing undefined"); 
  if (input->XNumVects == OSIV_IOINT_BLANK) OsivError("number of x vectors undefined"); 
  if (input->YNumVects == OSIV_IOINT_BLANK) OsivError("number of y vectors undefined"); 
  if (input->XWinSize == OSIV_IOINT_BLANK) OsivError("x window size undefined"); 
  if (input->YWinSize == OSIV_IOINT_BLANK) OsivError("y window size undefined"); 
  if (input->XMaxDisp == OSIV_IOINT_BLANK) OsivError("maximum x displacement undefined"); 
  if (input->YMaxDisp == OSIV_IOINT_BLANK) OsivError("maximum y displacement undefined"); 
  if (input->FrmStart == OSIV_IOINT_BLANK) OsivError("start frame undefined"); 
  if (input->FrmFinish == OSIV_IOINT_BLANK) OsivError("stop frame undefined"); 
  if (input->FrmSkip == OSIV_IOINT_BLANK) OsivError("frame jump undefined"); 
  if (input->FrmSet == OSIV_IOINT_BLANK) OsivError("frame spacing undefined"); 
  if (!strlen(input->MovieFN)) OsivError("movie file undefined"); 
  if (!strlen(input->VectorsFN)) OsivError("vectors file undefined"); 
 
  if (input->PreProc < 0 || input->PreProc > 5) OsivError("unrecognized preprocessing method"); 
  if (input->Interp < 0 || input->Interp > 4) OsivError("unrecognized interpolation method"); 
  if (input->XSize < 1) OsivError("image width must be non-zero, non-negative"); 
  if (input->YSize < 1) OsivError("image height must be non-zero, non-negative"); 
  if (input->XStart < 1) OsivError("x start position must be non-zero, non-negative"); 
  if (input->YStart < 1) OsivError("y start position must be non-zero, non-negative"); 
  if (input->XVectSpace < 1) OsivError("x spacing must be non-zero, non-negative"); 
  if (input->YVectSpace < 1) OsivError("y spacing must be non-zero, non-negative"); 
  if (input->XNumVects < 1) OsivError("number of x vectors must be non-zero, non-negative"); 
  if (input->YNumVects < 1) OsivError("number of y vectors must be non-zero, non-negative"); 
  if (input->XWinSize < 1) OsivError("window x size must be non-zero, non-negative"); 
  if (input->YWinSize < 1) OsivError("window y size must be non-zero, non-negative"); 
  if (input->XMaxDisp < 1) OsivError("maximum x displacement must be non-zero, non-negative"); 
  if (input->YMaxDisp < 1) OsivError("maximum y displacement must be non-zero, non-negative"); 
  if (input->FrmStart < 0) OsivError("start frame must be non-negative"); 
  if (input->FrmFinish < 0) OsivError("stop frame must be non-negative"); 
  if (input->FrmSkip < 1) OsivError("frame jump must be non-zero, non-negative"); 
  if (input->FrmSet < 1) OsivError("frame spacing must be non-zero, non-negative"); 
 
  if ((log(2*input->XMaxDisp+1)+log(2*input->YMaxDisp+1))/log(2) >= OSIV_DISP_BITS) 
    OsivError("max displacement exceeds that specified at compile time"); 
  if ((log(input->XWinSize)+log(input->YWinSize))/log(2) >= OSIV_WINDOW_BITS) 
    OsivError("window size exceeds that specified at compile time"); 
  if ((log(input->XSize)+log(input->YSize))/log(2) >= OSIV_FRAME_BITS) 
    OsivError("image size exceeds that specified at compile time"); 
  if (log(input->FrmFinish)/log(2) >= OSIV_MOVIE_BITS) 
    OsivError("movie length exceeds that specified at compile time"); 
 
  if (input->XStart + input->XOffset < input->XMaxDisp + 1) 
    OsivError("x start position is too small"); 
  if (input->YStart + input->YOffset < input->YMaxDisp + 1) 
    OsivError("y start position is too small"); 
  if (input->XSize < input->XStart + (input->XNumVects-1)* 
      input->XVectSpace+input->XWinSize) 
    OsivError("x interrogation domain exceeds limits of image"); 
  if (input->YSize < input->YStart + (input->YNumVects-1)* 
      input->YVectSpace+input->YWinSize) 
    OsivError("y interrogation domain exceeds limits of image"); 
  if (input->XSize < input->XStart + input->XOffset + input->XMaxDisp +  
      (input->XNumVects-1)*input->XVectSpace+input->XWinSize) 
    OsivError("x interrogation domain exceeds limits of image"); 
  if (input->YSize < input->YStart + input->YOffset + input->YMaxDisp +  
      (input->YNumVects-1)*input->YVectSpace+input->YWinSize) 
    OsivError("y interrogation domain exceeds limits of image"); 
  if (input->FrmFinish < input->FrmStart) 
    OsivError("last frame must follow first");   
} 
 
static void OsivAlg2_GetErrorInt (osiv_param* p, 
				  osiv_imcp* im0, 
				  osiv_imcp* im1, 
				  osiv_immf* imInt) 
{ 
  osiv_intf i,j,k,m; 
  osiv_intf xlen = (p->XNumVects-1)*p->XVectSpace+p->XWinSize+1; 
  osiv_intf ylen = (p->YNumVects-1)*p->YVectSpace+p->YWinSize+1; 
 
  /* Set first pixel */ 
  { 
    osiv_imsp d = *im0-*im1; 
    *imInt = d*d; 
  } 
 
  /* Integrate first row */ 
  for (i = 1; i < xlen; i++) 
    { 
      osiv_imsp d = *(im0+i)-*(im1+i); 
      *(imInt+i) = *(imInt+i-1)+d*d; 
    } 
 
  /* Integrate rest of map */ 
  for (i=xlen, j=p->XSize; i < ylen*xlen; i+=xlen, j+=p->XSize) 
    { 
      register osiv_imsp d; 
 
      d = *(im0+j)-*(im1+j); 
      *(imInt+i) = d*d; 
      for (k = 1+i, m = 1+j; k < xlen+i; k++, m++)  
	{ 
	  d = *(im0+m)-*(im1+m); 
	  *(imInt+k) = *(imInt+k-1)+d*d; 
	  *(imInt+k-1) += *(imInt+k-1-xlen); 
	} 
      *(imInt+k-1) += *(imInt+k-1-xlen); 
    } 
} 
 
static void OsivAlg2_GetErrorWind (osiv_param* p, 
				   osiv_immf* imInt, 
				   osiv_immw* map) 
{ 
  osiv_intf i,j; 
  osiv_intf xlen = (p->XNumVects-1)*p->XVectSpace+p->XWinSize+1; 
 
  for (i = 0; i < p->YNumVects*p->YVectSpace; i+=p->YVectSpace) 
    { 
      for (j = i*xlen; j < p->XNumVects*p->XVectSpace+i*xlen; j+=p->XVectSpace) 
	{ 
	  *map = *(imInt+j) + *(imInt+j+p->XWinSize+p->YWinSize*xlen) 
	    - *(imInt+j+p->XWinSize) - *(imInt+j+p->YWinSize*xlen); 
	  map += p->MapLength; 
	} 
    } 
} 
 
static void OsivAlg2_ConvertMap (osiv_param* p, 
				 osiv_ioreal* dest, 
				 osiv_immw* src) 
{ 
  osiv_intw i; 
  for (i = 0; i < p->MapLength; i++) 
    *(dest+i) = (osiv_ioreal)*(src+i); 
} 
 
static void OsivAlg2_ConvertVects (osiv_param* p, 
				   osiv_ioreal* dest, 
				   osiv_real* src) 
{ 
  if (p->SwapReal) 
    { 
      osiv_intf i; 
      for (i = 0; i < 2*p->NumVects; i++) 
	*(dest+i) = (osiv_ioreal)*(src+i); 
    } 
} 
 
void OsivAlg2 (osiv_param *p) 
{ 
  osiv_intm i; 
  osiv_imcp* im0; 
  osiv_imcp* im1; 
  osiv_immf* imInt; 
  osiv_immw* map; 
  osiv_real* vects; 
  osiv_ioreal* mapStd; 
  osiv_ioreal* vectsStd; 
 
  /* Allocate memory */ 
  im0 = (osiv_imcp*)OsivMalloc(p->ImageLength*sizeof(osiv_imcp)); 
  im1 = (osiv_imcp*)OsivMalloc(p->ImageLength*sizeof(osiv_imcp)); 
  imInt = (osiv_immf*)OsivMalloc(((p->XNumVects-1)*p->XVectSpace+p->XWinSize+1)* 
				 ((p->YNumVects-1)*p->YVectSpace+p->YWinSize+1)* 
				 sizeof(osiv_immf)); 
  map = (osiv_immw*)OsivMalloc(p->NumVects*p->MapLength*sizeof(osiv_immw)); 
  vects = (osiv_real*)OsivMalloc(2*p->NumVects*sizeof(osiv_real)); 
 
  if (!p->WriteMap) 
    mapStd = NULL; 
  else 
    mapStd = (osiv_ioreal*)OsivMalloc(p->MapLength*sizeof(osiv_ioreal)); 
   
  if (!p->SwapReal) 
    vectsStd = (osiv_ioreal*)vects; 
  else 
    vectsStd = (osiv_ioreal*)OsivMalloc(2*p->NumVects*sizeof(osiv_ioreal)); 
 
  /* Open body tag */ 
  OsivWriteTag(p->VectorsFile,OSIV_TAG_BODY); 
 
  /* Run through images */ 
  for (i = p->FrmStart; i <= p->FrmFinish; i += p->FrmSkip) 
    { 
      osiv_intf j,k; 
      osiv_immw* map_ptr = map; 
      osiv_real* vects_ptr = vects; 
      osiv_imcp* im0Start,*im1Start; 
 
      /* Read the ith image */ 
      if (p->FrmSkip != p->FrmSet || 
	  i == p->FrmStart) 
	{ 
	  OsivGetImage(p,i,im0); 
	  OsivPreProc(p,i,im0); 
	} 
      else 
	{ 
	  osiv_imcp* tmp = im1; 
	  im1 = im0; im0 = tmp; 
	} 
 
      /* Read the ith image pair */ 
      OsivGetImage(p,i+p->FrmSet,im1); 
      OsivPreProc(p,i+p->FrmSet,im1); 
  
      /* Open frame tag */ 
      OsivWriteTag(p->VectorsFile,OSIV_TAG_FRAME); 
 
      /* Write frame number */ 
      OsivWriteFrameNum(p->VectorsFile,i); 
 
      /* Set the start locations */ 
      im0Start = im0 + (p->XStart+p->XOffset-1)+ 
	(p->YStart+p->YOffset-1)*p->XSize; 
      im1Start = im1 + (p->XStart-1)+(p->YStart-1)*p->XSize; 
 
      /* Run through each displacement */ 
      for (j = -p->YMaxDisp; j <= p->YMaxDisp; j++) 
	{ 
	  for (k = -p->XMaxDisp+j*p->XSize; 
	       k <= p->XMaxDisp+j*p->XSize; k++) 
	    { 
	      /* Calculate the integral */ 
	      OsivAlg2_GetErrorInt(p,im0Start+k,im1Start,imInt); 
				 
	      /* Use the integral to calculate error */ 
	      OsivAlg2_GetErrorWind(p,imInt,map_ptr++); 
	    }	 
	} 
 
      /* Reset map pointer */ 
      map_ptr = map; 
 
      /* Find the minimum error value */ 
      for (j = 0; j < p->NumVects; j++) 
	{ 
	  osiv_intd ind,ix,iy; 
 
	  /* Find location of minima */ 
	  ind = OsivGetMinInd_IMMW(map_ptr,p->MapLength); 
	  ix = ind%p->XMapSize;   iy = ind/p->XMapSize; 
 
	  /* Calculate actual displacement */ 
	  *(vects_ptr) = ix + p->XOffset - p->XMaxDisp; 
	  *(vects_ptr+1) = iy + p->YOffset - p->YMaxDisp; 
 
	  /* Perform interpolation */ 
	  if (ix > 0 && ix < p->XMapSize-1 && 
	      iy > 0 && iy < p->YMapSize-1) 
	    OsivInterpMin_IMMW(p->Interp,map_ptr+ind, 
			       -1,1, 
			       -p->XMapSize,p->XMapSize, 
			       vects_ptr,vects_ptr+1); 
 
	  /* Write error map to file */ 
	  if (p->WriteMap) 
	    { 
	      OsivAlg2_ConvertMap(p,mapStd,map_ptr); 
	      OsivWriteTag(p->VectorsFile,OSIV_TAG_BOX); 
	      OsivWriteBoxNum(p->VectorsFile,j); 
	      OsivWriteMap(p->VectorsFile,p->MapLength,mapStd); 
	      OsivWriteTag(p->VectorsFile,OSIV_GAT_BOX); 
	    } 
 
	  vects_ptr += 2; 
	  map_ptr += p->MapLength; 
	} 
 
      /* Write vectors to file */ 
      OsivAlg2_ConvertVects(p,vectsStd,vects); 
      OsivWriteVectors(p->VectorsFile,2*p->NumVects,vectsStd); 
 
      /* Close frame tag */ 
      OsivWriteTag(p->VectorsFile,OSIV_GAT_FRAME); 
 
    } 
 
  /* Close body tag */ 
  OsivWriteTag(p->VectorsFile,OSIV_GAT_BODY); 
 
  /* Free memory */ 
  free(im0); 
  free(im1); 
  free(map); 
  free(imInt); 
  free(vects); 
 
  if (p->WriteMap) 
    free(mapStd); 
  if (p->SwapReal) 
    free(vectsStd); 
 
} 
 
 
 
