/* File: alg6.c 
  
* Copyright (c) 2002 James A. Strother 
* 
* This program is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2 of the License, or 
* (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public License 
* along with this program; if not, write to the Free Software 
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
* 
*/ 
 
#include "osiv.h" 
 
void OsivAlg6_CheckOpts (osiv_input* input) 
{ 
  if (input->PreProc == OSIV_IOINT_BLANK) OsivError("preprocessing method undefined"); 
  if (input->Interp == OSIV_IOINT_BLANK) OsivError("interpolation method undefined"); 
  if (input->XSize == OSIV_IOINT_BLANK) OsivError("image width undefined"); 
  if (input->YSize == OSIV_IOINT_BLANK) OsivError("image height undefined"); 
  if (input->XStart == OSIV_IOINT_BLANK) OsivError("x start position undefined"); 
  if (input->YStart == OSIV_IOINT_BLANK) OsivError("y start position undefined"); 
  if (input->XOffset == OSIV_IOINT_BLANK) OsivError("x offset undefined"); 
  if (input->YOffset == OSIV_IOINT_BLANK) OsivError("y offset undefined"); 
  if (input->XVectSpace == OSIV_IOINT_BLANK) OsivError("x spacing undefined"); 
  if (input->YVectSpace == OSIV_IOINT_BLANK) OsivError("y spacing undefined"); 
  if (input->XNumVects == OSIV_IOINT_BLANK) OsivError("number of x vectors undefined"); 
  if (input->YNumVects == OSIV_IOINT_BLANK) OsivError("number of y vectors undefined"); 
  if (input->XWinSize == OSIV_IOINT_BLANK) OsivError("x window size undefined"); 
  if (input->YWinSize == OSIV_IOINT_BLANK) OsivError("y window size undefined"); 
  if (input->XMaxDisp == OSIV_IOINT_BLANK) OsivError("maximum x displacement undefined"); 
  if (input->YMaxDisp == OSIV_IOINT_BLANK) OsivError("maximum y displacement undefined"); 
  if (input->FrmStart == OSIV_IOINT_BLANK) OsivError("start frame undefined"); 
  if (input->FrmFinish == OSIV_IOINT_BLANK) OsivError("stop frame undefined"); 
  if (input->FrmSkip == OSIV_IOINT_BLANK) OsivError("frame jump undefined"); 
  if (input->FrmSet == OSIV_IOINT_BLANK) OsivError("frame spacing undefined"); 
  if (!strlen(input->MovieFN)) OsivError("movie file undefined"); 
  if (!strlen(input->VectorsFN)) OsivError("vectors file undefined"); 
 
  if (input->PreProc < 0 || input->PreProc > 5) OsivError("unrecognized preprocessing method"); 
  if (input->Interp < 0 || input->Interp > 4) OsivError("unrecognized interpolation method"); 
  if (input->XSize < 1) OsivError("image width must be non-zero, non-negative"); 
  if (input->YSize < 1) OsivError("image height must be non-zero, non-negative"); 
  if (input->XStart < 0) OsivError("x start position must be non-negative"); 
  if (input->YStart < 0) OsivError("y start position must be non-negative"); 
  if (input->XOffset != 0) OsivError("x offset must be zero"); 
  if (input->YOffset != 0) OsivError("y offset must be zero"); 
  if (input->XVectSpace < 1) OsivError("x spacing must be non-zero, non-negative"); 
  if (input->YVectSpace < 1) OsivError("y spacing must be non-zero, non-negative"); 
  if (input->XNumVects < 1) OsivError("number of x vectors must be non-zero, non-negative"); 
  if (input->YNumVects < 1) OsivError("number of y vectors must be non-zero, non-negative"); 
  if (input->XWinSize < 1) OsivError("window x size must be non-zero, non-negative"); 
  if (input->YWinSize < 1) OsivError("window y size must be non-zero, non-negative"); 
  if (input->XMaxDisp < 1) OsivError("maximum x displacement must be non-zero, non-negative"); 
  if (input->YMaxDisp < 1) OsivError("maximum y displacement must be non-zero, non-negative"); 
  if (input->FrmStart < 0) OsivError("start frame must be non-negative"); 
  if (input->FrmFinish < 0) OsivError("stop frame must be non-negative"); 
  if (input->FrmSkip < 1) OsivError("frame jump must be non-zero, non-negative"); 
  if (input->FrmSet < 1) OsivError("frame spacing must be non-zero, non-negative"); 
 
  if ((log(2*input->XMaxDisp+1)+log(2*input->YMaxDisp+1))/log(2) >= OSIV_DISP_BITS) 
    OsivError("max displacement exceeds that specified at compile time"); 
  if ((log(2*(input->XWinSize/2+1))+log(input->YWinSize))/log(2) >= OSIV_WINDOW_BITS) 
    OsivError("window size exceeds that specified at compile time"); 
  if ((log(input->XSize)+log(input->YSize))/log(2) >= OSIV_FRAME_BITS) 
    OsivError("image size exceeds that specified at compile time"); 
  if (log(input->FrmFinish)/log(2) >= OSIV_MOVIE_BITS) 
    OsivError("movie length exceeds that specified at compile time"); 
 
  if (input->XStart + input->XOffset < 0) 
    OsivError("x start position is too small"); 
  if (input->YStart + input->YOffset < 0) 
    OsivError("y start position is too small"); 
  if (input->XSize < input->XStart + 
      (input->XNumVects-1)*input->XVectSpace+input->XWinSize) 
    OsivError("x interrogation domain exceeds limits of image"); 
  if (input->YSize < input->YStart + 
      (input->YNumVects-1)*input->YVectSpace+input->YWinSize) 
    OsivError("y interrogation domain exceeds limits of image"); 
  if (input->XSize < input->XStart + input->XOffset + 
      (input->XNumVects-1)*input->XVectSpace+input->XWinSize) 
    OsivError("x interrogation domain exceeds limits of image"); 
  if (input->YSize < input->YStart + input->YOffset + 
      (input->YNumVects-1)*input->YVectSpace+input->YWinSize) 
    OsivError("y interrogation domain exceeds limits of image"); 
  if (input->FrmFinish < input->FrmStart) 
    OsivError("last frame must follow first");   
  if (input->FrmSkip != input->FrmSet) 
    OsivError("frame spacing must equal frame jump"); 
} 
 
static void OsivAlg6_CopyWindow (osiv_param* p, 
				 osiv_imcp* im, 
				 fftw_real* ft) 
{ 
  osiv_intf i,j; 
 
  for (i = 0; i < p->YWinSize*p->XSize; i+=p->XSize) 
    { 
      for (j = i; j < p->XWinSize+i; j++) *ft++ = *(im+j); 
      for (j = p->XWinSize; j < p->XFFTSize; j++) *ft++ = 0; 
    } 
} 
 
static void OsivAlg6_CopyWindows (osiv_param* p, 
				  osiv_imcp* im, 
				  fftw_real* ft) 
{ 
  osiv_intf i,j; 
 
  im += p->XStart+p->YStart*p->XSize; 
 
  for (i = 0; i < p->YNumVects*p->YVectSpace; i+=p->YVectSpace) 
    { 
      for (j = i*p->XSize; j < p->XNumVects*p->XVectSpace+i*p->XSize; 
	   j+=p->XVectSpace) 
	{		 
	  OsivAlg6_CopyWindow(p,im+j,ft); 
	  ft += p->FFTLength; 
	} 
    } 
} 
 
static void OsivAlg6_XCorr (osiv_param* p, 
			    fftw_real* a, 
			    fftw_real* b) 
			 
{ 
  osiv_intf i; 
  osiv_intw j; 
 
  for (i = 0; i < p->NumVects; i++) 
    { 
      for (j = 0; j < p->FFTLength; j += 2) 
	{ 
	  register fftw_real tmp; 
	  tmp = *(a+j)**(b+j) + *(a+j+1)**(b+j+1); 
	  *(a+j+1) = *(a+j+1)**(b+j) - *(a+j)**(b+j+1); 
	  *(a+j) = tmp; 
	} 
      a += p->FFTLength; 
      b += p->FFTLength; 
    } 
} 
 
static void OsivAlg6_ConvertMap (osiv_param* p, 
				 osiv_ioreal* dest, 
				 fftw_real* src) 
{ 
  osiv_intw i,j; 
 
  for (i = p->YWinSize-p->YMaxDisp; i < p->YWinSize; i++) 
    { 
      j = p->XWinSize-p->XMaxDisp; 
      for (j += i*p->XFFTSize; j < p->XWinSize+i*p->XFFTSize; j++) 
	*dest++ = *(src+j); 
 
      for (j = i*p->XFFTSize; j <= p->XMaxDisp+i*p->XFFTSize; j++) 
	*dest++ = *(src+j); 
    } 
 
  for (i = 0; i <= p->YMaxDisp; i++) 
    { 
      j = p->XWinSize-p->XMaxDisp; 
      for (j += i*p->XFFTSize; j < p->XWinSize+i*p->XFFTSize; j++) 
	*dest++ = *(src+j); 
 
      for (j = i*p->XFFTSize; j <= p->XMaxDisp+i*p->XFFTSize; j++) 
	*dest++ = *(src+j); 
    } 
} 
 
static void OsivAlg6_ConvertVects (osiv_param* p, 
				   osiv_ioreal* dest, 
				   osiv_real* src) 
{ 
  if (p->SwapReal) 
    { 
      osiv_intf i; 
      for (i = 0; i < 2*p->NumVects; i++) 
	*(dest+i) = (osiv_ioreal)*(src+i); 
    } 
} 
 
void OsivAlg6 (osiv_param *p) 
{ 
  osiv_intm i; 
  osiv_imcp *im; 
  fftw_real *ft0,*ft1; 
  fftw_plan forw,back; 
  osiv_real* vects; 
  osiv_ioreal* mapStd; 
  osiv_ioreal* vectsStd; 
  int *array_dims; 
 
  /* Allocate memory */ 
  im = (osiv_imcp*)OsivMalloc(p->ImageLength*sizeof(osiv_imcp)); 
  vects = (osiv_real*)OsivMalloc(2*p->NumVects*sizeof(osiv_real)); 
  ft0 = (fftw_real*)fftw_malloc(p->NumVects*p->FFTLength*sizeof(fftw_real)); 
  ft1 = (fftw_real*)fftw_malloc(p->NumVects*p->FFTLength*sizeof(fftw_real)); 
  if (!ft0 || !ft1) OsivError("unable to allocate memory"); 
 
  if (!p->WriteMap) 
    mapStd = NULL; 
  else 
    mapStd = (osiv_ioreal*)OsivMalloc(p->MapLength*sizeof(osiv_ioreal)); 
 
  if (!p->SwapReal) 
    vectsStd = (osiv_ioreal*)vects; 
  else 
    vectsStd = (osiv_ioreal*)OsivMalloc(2*p->NumVects*sizeof(osiv_ioreal)); 
 
  /* Generate fftw array dimensions */ 
  array_dims = (int*)OsivMalloc(2*sizeof(int)); 
  array_dims[0] = p->YWinSize; 
  array_dims[1] = p->XWinSize; 
 
  /* Generate fftw plans */ 
  forw = fftw_plan_many_dft_r2c(2,array_dims,p->NumVects, 
				ft0,NULL,1,p->FFTLength, 
				(fftw_complex*)ft0,NULL,1,p->FFTLength/2, 
				FFTW_ESTIMATE); 
 
  if (!forw) OsivError("unable to complete fftw plan"); 
 
  /* Read the first image */ 
  OsivGetImage(p,p->FrmStart,im); 
  OsivPreProc(p,p->FrmStart,im); 
  OsivAlg6_CopyWindows(p,im,ft0); 
  fftw_execute(forw); 
  fftw_destroy_plan(forw); 
 
  /* Open body tag */ 
  OsivWriteTag(p->VectorsFile,OSIV_TAG_BODY); 
   
  /* Run through remainder of images */ 
  for (i = p->FrmStart; i <= p->FrmFinish; i += p->FrmSkip) 
    { 
      osiv_intf j; 
      fftw_real* ft_ptr = ft0; 
      osiv_real* vects_ptr = vects; 
 
      /* Generate fftw plans */ 
      forw = fftw_plan_many_dft_r2c(2,array_dims,p->NumVects, 
				    ft1,NULL,1,p->FFTLength, 
				    (fftw_complex*)ft1,NULL,1,p->FFTLength/2, 
				    FFTW_ESTIMATE); 
      back = fftw_plan_many_dft_c2r(2,array_dims,p->NumVects, 
				    (fftw_complex*)ft0,NULL,1,p->FFTLength/2, 
				    ft0,NULL,1,p->FFTLength, 
				    FFTW_ESTIMATE); 
 
      if (!forw || !back) OsivError("unable to complete fftw plan"); 
 
      /* Read the ith image pair */ 
      OsivGetImage(p,i+p->FrmSet,im); 
      OsivPreProc(p,i+p->FrmSet,im); 
 
      /* Get ffts for all windows */ 
      OsivAlg6_CopyWindows(p,im,ft1); 
      fftw_execute(forw); 
      fftw_destroy_plan(forw); 
 
      /* Perform in-place correlation */ 
      OsivAlg6_XCorr(p,ft0,ft1); 
 
      /* Calculate the inverse fft */ 
      fftw_execute(back); 
      fftw_destroy_plan(back); 
 
      /* Open frame tag */ 
      OsivWriteTag(p->VectorsFile,OSIV_TAG_FRAME); 
 
      /* Write frame number */ 
      OsivWriteFrameNum(p->VectorsFile,i); 
 
      /* Run through each of the windows */ 
      for (j = 0; j < p->NumVects; j++) 
	{ 
	  osiv_intd ind,ix,iy; 
 
	  /* Find the maximum value */ 
	  ind = OsivGetMaxInd_FFT(ft_ptr, 
				  p->XWinSize,p->YWinSize, 
				  p->XMaxDisp,p->YMaxDisp, 
				  p->XFFTSize); 
 
	  ix = ind%p->XFFTSize; iy = ind/p->XFFTSize; 
	   
	  /* Calculate actual displacement */ 
	  *vects_ptr = ix + p->XOffset  
	    - p->XWinSize*(ix/(p->XWinSize/2+1)); 
	  *(vects_ptr+1) = iy + p->YOffset  
	    - p->YWinSize*(iy/(p->YWinSize/2+1)); 
        
	  /* Perform interpolation */ 
	  if (ix != p->XWinSize/2 && iy != p->YWinSize/2) 
	    { 
	      osiv_intd lft,rht,up,dwn; 
 
	      if (ix != 0) lft = -1; 
	      else lft = p->XWinSize-1; 
 
	      if (ix != p->XWinSize-1) rht = 1; 
	      else rht = -(p->XWinSize-1); 
 
	      if (iy != 0) up = -p->XFFTSize; 
	      else up = (p->YWinSize-1)*p->XFFTSize; 
 
	      if (iy != p->YWinSize-1) dwn = p->XFFTSize; 
	      else dwn = -(p->YWinSize-1)*p->XFFTSize; 
 
	      OsivInterpMax_FFTW(p->Interp, 
				 ft_ptr+ind,  
				 lft,rht, 
				 up,dwn, 
				 vects_ptr,vects_ptr+1); 
	    } 
 
	  /* Write error map to file */ 
	  if (p->WriteMap) 
	    { 
	      OsivAlg6_ConvertMap(p,mapStd,ft_ptr); 
	      OsivWriteTag(p->VectorsFile,OSIV_TAG_BOX); 
	      OsivWriteBoxNum(p->VectorsFile,j); 
	      OsivWriteMap(p->VectorsFile,p->MapLength,mapStd); 
	      OsivWriteTag(p->VectorsFile,OSIV_GAT_BOX); 
	    } 
 
	  vects_ptr += 2; 
	  ft_ptr += p->FFTLength; 
	} 
 
      /* Write vectors to file */ 
      OsivAlg6_ConvertVects(p,vectsStd,vects); 
      OsivWriteVectors(p->VectorsFile,2*p->NumVects,vectsStd); 
 
      /* Close frame tag */ 
      OsivWriteTag(p->VectorsFile,OSIV_GAT_FRAME); 
 
      /* Swap FFTs */ 
      {fftw_real*tmp=ft1;ft1=ft0;ft0=tmp;} 
    } 
 
  /* Close body tag */ 
  OsivWriteTag(p->VectorsFile,OSIV_GAT_BODY); 
 
  /* Free memory */ 
  free(im); 
  free(vects); 
  free(array_dims); 
  fftw_free(ft0); 
  fftw_free(ft1); 
 
  if (p->WriteMap) 
    free(mapStd); 
  if (p->SwapReal) 
    free(vectsStd); 
} 
