/* File: alg8.c 
 
* Copyright (c) 2002 James A. Strother 
* 
* This program is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2 of the License, or 
* (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public License 
* along with this program; if not, write to the Free Software 
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
* 
*/ 
 
#include "osiv.h" 
 
void OsivAlg8_CheckOpts (osiv_input* input) 
{ 
  if (input->PreProc == OSIV_IOINT_BLANK) OsivError("preprocessing method undefined"); 
  if (input->Interp == OSIV_IOINT_BLANK) OsivError("interpolation method undefined"); 
  if (input->XSize == OSIV_IOINT_BLANK) OsivError("image width undefined"); 
  if (input->YSize == OSIV_IOINT_BLANK) OsivError("image height undefined"); 
  if (input->XStart == OSIV_IOINT_BLANK) OsivError("x start position undefined"); 
  if (input->YStart == OSIV_IOINT_BLANK) OsivError("y start position undefined"); 
  if (input->XOffset == OSIV_IOINT_BLANK) OsivError("x offset undefined"); 
  if (input->YOffset == OSIV_IOINT_BLANK) OsivError("y offset undefined"); 
  if (input->XVectSpace == OSIV_IOINT_BLANK) OsivError("x spacing undefined"); 
  if (input->YVectSpace == OSIV_IOINT_BLANK) OsivError("y spacing undefined"); 
  if (input->XNumVects == OSIV_IOINT_BLANK) OsivError("number of x vectors undefined"); 
  if (input->YNumVects == OSIV_IOINT_BLANK) OsivError("number of y vectors undefined"); 
  if (input->XWinSize == OSIV_IOINT_BLANK) OsivError("x window size undefined"); 
  if (input->YWinSize == OSIV_IOINT_BLANK) OsivError("y window size undefined"); 
  if (input->FrmStart == OSIV_IOINT_BLANK) OsivError("start frame undefined"); 
  if (input->FrmFinish == OSIV_IOINT_BLANK) OsivError("stop frame undefined"); 
  if (input->FrmSkip == OSIV_IOINT_BLANK) OsivError("frame jump undefined"); 
  if (input->FrmSet == OSIV_IOINT_BLANK) OsivError("frame spacing undefined"); 
  if (input->NumIter == OSIV_IOINT_BLANK) OsivError("number of iterations undefined"); 
  if (!strlen(input->MovieFN)) OsivError("movie file undefined"); 
  if (!strlen(input->VectorsFN)) OsivError("vectors file undefined"); 
 
  if (input->PreProc < 0 || input->PreProc > 5) OsivError("unrecognized preprocessing method"); 
  if (input->Interp < 0 || input->Interp > 4) OsivError("unrecognized interpolation method"); 
  if (input->XSize < 1) OsivError("image width must be non-zero, non-negative"); 
  if (input->YSize < 1) OsivError("image height must be non-zero, non-negative"); 
  if (input->XStart < 0) OsivError("x start position must be non-negative"); 
  if (input->YStart < 0) OsivError("y start position must be non-negative"); 
  if (input->XVectSpace < 1) OsivError("x spacing must be non-zero, non-negative"); 
  if (input->YVectSpace < 1) OsivError("y spacing must be non-zero, non-negative"); 
  if (input->XNumVects < 1) OsivError("number of x vectors must be non-zero, non-negative"); 
  if (input->YNumVects < 1) OsivError("number of y vectors must be non-zero, non-negative"); 
  if (input->XWinSize < 1) OsivError("window x size must be non-zero, non-negative"); 
  if (input->YWinSize < 1) OsivError("window y size must be non-zero, non-negative"); 
  if (input->FrmStart < 0) OsivError("start frame must be non-negative"); 
  if (input->FrmFinish < 0) OsivError("stop frame must be non-negative"); 
  if (input->FrmSkip < 1) OsivError("frame jump must be non-zero, non-negative"); 
  if (input->FrmSet < 1) OsivError("frame spacing must be non-zero, non-negative"); 
  if (input->NumIter < 1) OsivError("number of iterations must non-zero, non-negative"); 
 
  if ((log(2*input->XMaxDisp+1)+log(2*input->YMaxDisp+1))/log(2) >= OSIV_DISP_BITS) 
    OsivError("max displacement exceeds that specified at compile time"); 
  if ((log(2*(input->XWinSize/2+1))+log(input->YWinSize))/log(2) >= OSIV_WINDOW_BITS) 
    OsivError("window size exceeds that specified at compile time"); 
  if ((log(input->XSize)+log(input->YSize))/log(2) >= OSIV_FRAME_BITS) 
    OsivError("image size exceeds that specified at compile time"); 
  if (log(input->FrmFinish)/log(2) >= OSIV_MOVIE_BITS) 
    OsivError("movie length exceeds that specified at compile time"); 
 
  if (input->XStart + input->XOffset < input->XMaxDisp) 
    OsivError("x start position is too small"); 
  if (input->YStart + input->YOffset < input->YMaxDisp) 
    OsivError("y start position is too small"); 
  if (input->XSize < input->XStart + 
      (input->XNumVects-1)*input->XVectSpace+input->XWinSize) 
    OsivError("x interrogation domain exceeds limits of image"); 
  if (input->YSize < input->YStart + 
      (input->YNumVects-1)*input->YVectSpace+input->YWinSize) 
    OsivError("y interrogation domain exceeds limits of image"); 
  if (input->XSize < input->XStart + input->XOffset + input->XMaxDisp +  
      (input->XNumVects-1)*input->XVectSpace+input->XWinSize) 
    OsivError("x interrogation domain exceeds limits of image"); 
  if (input->YSize < input->YStart + input->YOffset + input->YMaxDisp + 
      (input->YNumVects-1)*input->YVectSpace+input->YWinSize) 
    OsivError("y interrogation domain exceeds limits of image"); 
  if (input->FrmFinish < input->FrmStart) 
    OsivError("last frame must follow first");   
  if (input->NumIter < 2) 
    OsivError("number of iterations must be above one"); 
  if (input->WriteMap == 1) 
    OsivError("unable to keep map for iterative methods"); 
} 
 
static void OsivAlg8_CopyWindow (osiv_param* p, 
				 osiv_imcp* im, 
				 fftw_real* ft) 
{ 
  osiv_intf i,j; 
 
  for (i = 0; i < p->YWinSize*p->XSize; i+=p->XSize) 
    { 
      for (j = i; j < p->XWinSize+i; j++) *ft++ = *(im+j); 
      for (j = p->XWinSize; j < p->XFFTSize; j++) *ft++ = 0; 
    } 
} 
 
static void OsivAlg8_XCorr (fftw_real* a, 
			    fftw_real* b, 
			    osiv_intw len) 
{ 
  osiv_intw i; 
 
  for (i = 0; i < len; i += 2) 
    { 
      register fftw_real tmp; 
      tmp = *(a+i)**(b+i) + *(a+i+1)**(b+i+1); 
      *(a+i+1) = *(a+i+1)**(b+i) - *(a+i)**(b+i+1); 
      *(a+i) = tmp; 
    } 
} 
 
static void OsivAlg8_ConvertVects (osiv_param* p, 
				   osiv_ioreal* dest, 
				   osiv_real* src) 
{ 
  if (p->SwapReal) 
    { 
      osiv_intf i; 
      for (i = 0; i < 2*p->NumVects; i++) 
	*(dest+i) = (osiv_ioreal)*(src+i); 
    } 
} 
 
void OsivAlg8 (osiv_param *p) 
{ 
  osiv_intm i; 
  osiv_imcp* im0,*im1; 
  fftw_real* ft0,*ft1; 
  osiv_real* vects; 
  fftw_plan forw0,forw1; 
  fftw_plan back0; 
  osiv_ioreal* vectsStd; 
 
  /* Allocate memory */ 
  im0 = (osiv_imcp*)OsivMalloc(p->ImageLength*sizeof(osiv_imcp)); 
  im1 = (osiv_imcp*)OsivMalloc(p->ImageLength*sizeof(osiv_imcp)); 
  vects = (osiv_real*)OsivMalloc(2*p->NumVects*sizeof(osiv_real)); 
  ft0 = (fftw_real*)fftw_malloc(p->FFTLength*sizeof(fftw_real)); 
  ft1 = (fftw_real*)fftw_malloc(p->FFTLength*sizeof(fftw_real)); 
  if (!ft0 || !ft1) OsivError("unable to allocate memory"); 
 
  if (!p->SwapReal) 
    vectsStd = (osiv_ioreal*)vects; 
  else 
    vectsStd = (osiv_ioreal*)OsivMalloc(2*p->NumVects*sizeof(osiv_ioreal)); 
 
  /* Plan FFTs */ 
  forw0 = fftw_plan_dft_r2c_2d(p->YWinSize,p->XWinSize,ft0,(fftw_complex*)ft0,FFTW_ESTIMATE); 
  forw1 = fftw_plan_dft_r2c_2d(p->YWinSize,p->XWinSize,ft1,(fftw_complex*)ft1,FFTW_ESTIMATE); 
  back0 = fftw_plan_dft_c2r_2d(p->YWinSize,p->XWinSize,(fftw_complex*)ft0,ft0,FFTW_ESTIMATE); 
  if (!forw0 || !forw1 || !back0) OsivError("unable to complete fftw plan"); 
 
  /* Open body tag */ 
  OsivWriteTag(p->VectorsFile,OSIV_TAG_BODY); 
   
  /* Run through remainder of images */ 
  for (i = p->FrmStart; i <= p->FrmFinish; i += p->FrmSkip) 
    { 
      osiv_intf j,k; 
      osiv_imcp* im0Start,*im1Start; 
      osiv_real* vects_ptr = vects; 
 
      /* Read the ith image */ 
      if (p->FrmSkip != p->FrmSet || 
	  i == p->FrmStart) 
	{ 
	  OsivGetImage(p,i,im0); 
	  OsivPreProc(p,i,im0); 
	} 
      else 
	{ 
	  osiv_imcp* tmp = im1; 
	  im1 = im0; im0 = tmp; 
	} 
 
      /* Read the ith image pair */ 
      OsivGetImage(p,i+p->FrmSet,im1); 
      OsivPreProc(p,i+p->FrmSet,im1); 
 
      /* Open frame tag */ 
      OsivWriteTag(p->VectorsFile,OSIV_TAG_FRAME); 
 
      /* Write frame number */ 
      OsivWriteFrameNum(p->VectorsFile,i); 
 
      /* Set the start locations */ 
      im0Start = im0 + p->XStart+p->XOffset+ 
	(p->YStart+p->YOffset)*p->XSize; 
      im1Start = im1 + p->XStart+p->YStart*p->XSize; 
 
      /* Run through each of the windows */ 
      for (j = 0; j < p->YNumVects; j++) 
	{ 
	  for (k = 0; k < p->XNumVects; k++) 
	    { 
	      osiv_intf pos; 
	      osiv_intd m=0,ind=0; 
	      osiv_intd dx=0,dy=0; 
	      osiv_intd ix=0,iy=0; 
 
	      /* Set position of window */ 
	      pos = k*p->XVectSpace 
		+j*p->YVectSpace*p->XSize; 
 
	      /* Begin iterations */ 
	      while (m < p->NumIter &&  
		     !(m!=0 && dx==0 && dy==0) && 
		     abs(dx) < p->XMaxDisp && 
		     abs(dy) < p->YMaxDisp) 
		{ 
		  /* Copy windows */  
		  OsivAlg8_CopyWindow(p,im0Start+pos+dx+dy*p->XSize,ft0); 
		  OsivAlg8_CopyWindow(p,im1Start+pos,ft1); 
 
		  /* Calculate ffts */ 
		  fftw_execute(forw0); 
		  fftw_execute(forw1); 
 
		  /* Perform an in-place correlation */ 
		  OsivAlg8_XCorr(ft0,ft1,p->FFTLength); 
			 
		  /* Calculate the inverse fft */ 
		  fftw_execute(back0); 
			 
		  /* Find the maximum value */ 
		  ind = OsivGetMaxInd_FFT(ft0, 
					  p->XWinSize,p->YWinSize, 
					  p->XMaxDisp,p->YMaxDisp, 
					  p->XFFTSize); 
 
		  ix = ind%p->XFFTSize; iy = ind/p->XFFTSize; 
 
		  /* Set new displacement */ 
		  dx += ix - p->XWinSize*(ix/(p->XWinSize/2+1)); 
		  dy += iy - p->XWinSize*(iy/(p->XWinSize/2+1)); 
 
		  m++; 
		} 
 
	      /* Convert displacements to vectors */ 
	      *vects_ptr = dx + p->XOffset; 
	      *(vects_ptr+1) = dy + p->YOffset; 
 
	      /* Perform interpolation */ 
	      if (ix != p->XWinSize/2  && 
		  iy != p->YWinSize/2) 
		{ 
		  osiv_intd lft,rht,up,dwn; 
 
		  if (ix != 0) lft = -1; 
		  else lft = p->XWinSize-1; 
 
		  if (ix != p->XWinSize-1) rht = 1; 
		  else rht = -(p->XWinSize-1); 
 
		  if (iy != 0) up = -p->XFFTSize; 
		  else up = (p->YWinSize-1)*p->XFFTSize; 
 
		  if (iy != p->YWinSize-1) dwn = p->XFFTSize; 
		  else dwn = -(p->YWinSize-1)*p->XFFTSize; 
 
		  OsivInterpMax_FFTW(p->Interp, 
				     ft0+ind,  
				     lft,rht, 
				     up,dwn, 
				     vects_ptr,vects_ptr+1); 
		} 
 
	      vects_ptr += 2; 
	    } 
	} 
 
      /* Write vectors to file */ 
      OsivAlg8_ConvertVects(p,vectsStd,vects); 
      OsivWriteVectors(p->VectorsFile,2*p->NumVects,vectsStd); 
 
      /* Close frame tag */ 
      OsivWriteTag(p->VectorsFile,OSIV_GAT_FRAME); 
 
    } 
 
  /* Close body tag */ 
  OsivWriteTag(p->VectorsFile,OSIV_GAT_BODY); 
 
  /* Kill plan */ 
  fftw_destroy_plan(forw0); 
  fftw_destroy_plan(forw1); 
  fftw_destroy_plan(back0); 
 
  /* Free memory */ 
  free(im0); 
  free(im1); 
  free(vects); 
  fftw_free(ft0); 
  fftw_free(ft1); 
 
  if (p->SwapReal) 
    free(vectsStd); 
} 
