/* File: interp.c 
 
 * Copyright (c) 2002 James A. Strother 
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 * 
*/ 
 
#include "osiv.h" 
 
static void OsivInterpMin_Method1 (osiv_real lftval, 
				   osiv_real midval, 
				   osiv_real rhtval, 
				   osiv_real *x) 
{ 
  *x += .5*log(lftval/rhtval)/log(lftval*rhtval/(midval*midval)); 
} 
 
static void OsivInterpMin_Method2 (osiv_real lftval, 
				   osiv_real midval, 
				   osiv_real rhtval, 
				   osiv_real *x) 
{ 
  *x += .5*(lftval-rhtval)/(lftval+rhtval-2*midval); 
} 
 
static void OsivInterpMin_Method3 (osiv_real v0, osiv_real v1, osiv_real v2, 
				   osiv_real v3, osiv_real v4, osiv_real v5, 
				   osiv_real v6, osiv_real v7, osiv_real v8, 
				   osiv_real *x, osiv_real *y) 
{ 
  OsivInterpMin_Method2(v0+v3+v6,v1+v4+v7,v2+v5+v8,x); 
  OsivInterpMin_Method2(v0+v1+v2,v3+v4+v5,v6+v7+v8,y); 
} 
 
static void OsivInterpMin_Method4 (osiv_real lftval, 
				   osiv_real midval, 
				   osiv_real rhtval, 
				   osiv_real* x) 
{ 
  *x += (lftval-rhtval)/(lftval+midval+rhtval); 
} 
 
static void OsivInterpMax_Method1 (osiv_real lftval, 
				   osiv_real midval, 
				   osiv_real rhtval, 
				   osiv_real *x) 
{ 
  *x += .5*log(lftval/rhtval)/log(lftval*rhtval/(midval*midval)); 
} 
 
static void OsivInterpMax_Method2 (osiv_real lftval, 
				   osiv_real midval, 
				   osiv_real rhtval, 
				   osiv_real *x) 
{ 
  *x += .5*(lftval-rhtval)/(lftval+rhtval-2*midval); 
} 
 
static void OsivInterpMax_Method3 (osiv_real v0, osiv_real v1, osiv_real v2, 
				   osiv_real v3, osiv_real v4, osiv_real v5, 
				   osiv_real v6, osiv_real v7, osiv_real v8, 
				   osiv_real *x, osiv_real *y) 
{ 
  OsivInterpMax_Method2(v0+v3+v6,v1+v4+v7,v2+v5+v8,x); 
  OsivInterpMax_Method2(v0+v1+v2,v3+v4+v5,v6+v7+v8,y); 
} 
 
static void OsivInterpMax_Method4 (osiv_real lftval, 
				   osiv_real midval, 
				   osiv_real rhtval, 
				   osiv_real* x) 
{ 
  *x += (rhtval-lftval)/(lftval+midval+rhtval); 
} 
 
void OsivInterpMin_IMCP (osiv_inta method, 
			 osiv_imcp *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMin_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMin_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMin_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMin_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMin_IMSP (osiv_inta method, 
			 osiv_imsp *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMin_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMin_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMin_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMin_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMin_IMMP (osiv_inta method, 
			 osiv_immp *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMin_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMin_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMin_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMin_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMin_IMCW (osiv_inta method, 
			 osiv_imcw *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMin_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMin_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMin_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMin_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMin_IMSW (osiv_inta method, 
			 osiv_imsw *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMin_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMin_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMin_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMin_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMin_IMMW (osiv_inta method, 
			 osiv_immw *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMin_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMin_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMin_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMin_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMin_IMCF (osiv_inta method, 
			 osiv_imcf *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMin_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMin_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMin_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMin_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMin_IMSF (osiv_inta method, 
			 osiv_imsf *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMin_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMin_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMin_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMin_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMin_IMMF (osiv_inta method, 
			 osiv_immf *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMin_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMin_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMin_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMin_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
 
void OsivInterpMin_REAL (osiv_inta method, 
			 osiv_real *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMin_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMin_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMin_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMin_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMin_FFTW (osiv_inta method, 
			 fftw_real *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMin_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMin_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMin_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMin_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMin_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMax_IMCP (osiv_inta method, 
			 osiv_imcp *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMax_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMax_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMax_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMax_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMax_IMSP (osiv_inta method, 
			 osiv_imsp *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMax_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMax_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMax_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMax_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMax_IMMP (osiv_inta method, 
			 osiv_immp *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMax_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMax_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMax_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMax_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMax_IMCW (osiv_inta method, 
			 osiv_imcw *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMax_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMax_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMax_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMax_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMax_IMSW (osiv_inta method, 
			 osiv_imsw *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMax_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMax_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMax_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMax_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMax_IMMW (osiv_inta method, 
			 osiv_immw *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMax_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMax_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMax_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMax_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMax_IMCF (osiv_inta method, 
			 osiv_imcf *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMax_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMax_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMax_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMax_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMax_IMSF (osiv_inta method, 
			 osiv_imsf *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMax_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMax_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMax_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMax_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMax_IMMF (osiv_inta method, 
			 osiv_immf *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMax_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMax_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMax_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMax_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMax_REAL (osiv_inta method, 
			 osiv_real *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMax_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMax_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMax_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMax_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
 
void OsivInterpMax_FFTW (osiv_inta method, 
			 fftw_real *ind, 
			 osiv_intf lft, 
			 osiv_intf rht, 
			 osiv_intf up, 
			 osiv_intf dwn, 
			 osiv_real *x, 
			 osiv_real *y) 
{		   
  switch (method) 
    { 
    case 0: 
      break; 
    case 1: 
      OsivInterpMax_Method1(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method1(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 2: 
      OsivInterpMax_Method2(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method2(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    case 3: 
      OsivInterpMax_Method3(*(ind+up+lft),*(ind+up),*(ind+up+rht), 
			    *(ind+lft),*(ind),*(ind+rht), 
			    *(ind+dwn+lft),*(ind+dwn),*(ind+dwn+rht), 
			    x,y); 
      break; 
    case 4: 
      OsivInterpMax_Method4(*(ind+lft),*(ind),*(ind+rht),x); 
      OsivInterpMax_Method4(*(ind+up),*(ind),*(ind+dwn), y); 
      break; 
    } 
} 
