/* File: osiv.c 
 
* Copyright (c) 2002 James A. Strother 
* 
* This program is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2 of the License, or 
* (at your option) any later version. 
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details. 
* 
* You should have received a copy of the GNU General Public License 
* along with this program; if not, write to the Free Software 
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
* 
*/ 
 
#include "osiv.h" 
 
void OsivError (const char* m) 
{ 
  fprintf(stderr, "%s error: %s\n",OSIV_PACKAGE_NAME,m); 
  exit(EXIT_FAILURE); 
} 
 
void OsivError2(const char* m1,	const char* m2) 
{ 
  fprintf(stderr, "%s error: %s%s\n",OSIV_PACKAGE_NAME,m1,m2); 
  exit(EXIT_FAILURE); 
} 
 
void* OsivCalloc(size_t nmemb, size_t size) 
{ 
  char* ptr; 
 
  ptr = (char*)calloc(nmemb,size); 
  if (ptr == NULL) 
    OsivError("unable to allocate memory"); 
  return (void*)ptr; 
} 
 
void* OsivMalloc(size_t nbytes) 
{ 
  char* ptr; 
 
  ptr = (char*)malloc(nbytes); 
  if (ptr == NULL) 
    OsivError("unable to allocate memory"); 
  return (void*)ptr; 
} 
 
int main (int argc, char** argv) 
{ 
  osiv_input input; 
  osiv_param p; 
   
  /* Parse input from command line */ 
  OsivParseInput(&input,argc-1,argv+1); 
 
  /* Open data file */ 
  if (!strcmp(input.VectorsFN,"-")) 
    p.VectorsFile = stdout; 
  else if (!(p.VectorsFile = fopen(input.VectorsFN,"wb"))) 
    OsivError("can't open output file"); 
 
  /* Write parameters to file */ 
  OsivWriteHeader(p.VectorsFile,&input); 
 
  /* Copy over scalars */ 
  p.PreProc = input.PreProc; 
  p.Interp = input.Interp; 
  p.XSize = input.XSize; 
  p.YSize = input.YSize; 
  p.XStart = input.XStart; 
  p.YStart = input.YStart; 
  p.XOffset = input.XOffset; 
  p.YOffset = input.YOffset; 
  p.XVectSpace = input.XVectSpace; 
  p.YVectSpace = input.YVectSpace; 
  p.XNumVects = input.XNumVects; 
  p.YNumVects = input.YNumVects; 
  p.XWinSize = input.XWinSize; 
  p.YWinSize = input.YWinSize; 
  p.XMaxDisp = input.XMaxDisp; 
  p.YMaxDisp = input.YMaxDisp; 
  p.XWinLgSize = input.XWinLgSize; 
  p.YWinLgSize = input.YWinLgSize; 
  p.FrmStart = input.FrmStart; 
  p.FrmFinish = input.FrmFinish; 
  p.FrmSkip = input.FrmSkip; 
  p.FrmSet = input.FrmSet; 
  p.NumIter = input.NumIter; 
  p.NumThreads = input.NumThreads; 
  p.WriteMap = (input.WriteMap == 1); 
 
  /* Calculate some useful numbers */ 
  p.XMapSize = 2*p.XMaxDisp+1; 
  p.YMapSize = 2*p.YMaxDisp+1; 
  p.MapLength = p.XMapSize*p.YMapSize; 
 
  p.XFFTSize = 2*(p.XWinSize/2+1); 
  p.YFFTSize = p.YWinSize; 
  p.FFTLength = p.XFFTSize*p.YFFTSize; 
 
  p.XFFTLgSize = 2*(p.XWinLgSize/2+1); 
  p.YFFTLgSize = p.YWinLgSize; 
  p.FFTLgLength = p.XFFTLgSize*p.YFFTLgSize; 
 
  p.WinLength = p.XWinSize*p.YWinSize; 
  p.ImageLength = p.XSize*p.YSize; 
  p.NumVects = p.XNumVects*p.YNumVects; 
 
  p.SwapReal = (sizeof(osiv_real)!=sizeof(osiv_ioreal)); 
 
  /* Create a file instance of movie */ 
  OsivSetupMovie(&p,input.MovieFN); 
  OsivSetupPreProc(&p); 
 
  /* Run Correlation */ 
  switch(input.Alg) 
    {  
    case 1: OsivAlg1(&p); break; 
    case 2: OsivAlg2(&p); break; 
    case 3: OsivAlg3(&p); break; 
    case 4: OsivAlg4(&p); break; 
    case 5: OsivAlg5(&p); break; 
    case 6: OsivAlg6(&p); break; 
    case 7: OsivAlg7(&p); break; 
    case 8: OsivAlg8(&p); break; 
#ifndef OSIV_NO_THREADS 
    case 9: OsivAlg9(&p); break; 
#endif 
    default: 
      OsivError("unrecognized algorithm");break; 
    } 
 
  /* Close movie file */ 
  OsivCloseMovie(&p); 
 
  /* Close preproc data */ 
  OsivClosePreProc(&p); 
 
  /* Close data file */ 
  if (p.VectorsFile == stdout) 
    fflush(p.VectorsFile); 
  else if (fclose(p.VectorsFile)) 
    OsivError("unable to close file"); 
 
  /* Clean up variables */ 
  free(input.MovieFN); 
  free(input.VectorsFN); 
 
  /* Exit with success */ 
  return EXIT_SUCCESS; 
} 
