/* File: osiv.h 
 
 * Copyright (c) 2002 James A. Strother 
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 * 
*/ 
 
#ifndef _OSIV_H 
 
/*  
 *   Put together includes  
 *  ****************************************************** 
*/ 
 
#include <stdio.h> 
#if HAVE_CONFIG_H 
# include "config.h" 
#endif 
#if HAVE_SYS_TYPE_H 
# include <sys/types.h> 
#endif 
#if STDC_HEADERS 
# include <stdlib.h> 
# include <stddef.h> 
# include <stdarg.h> 
#else 
# if HAVE_STDLIB_H 
#  include <stdlib.h> 
# endif 
# if HAVE_STDARG_H 
#  include <stdarg.h> 
# endif 
#endif 
#if HAVE_LIMITS_H 
# include <limits.h> 
#endif 
#if HAVE_STRING_H 
# ifdef STDC_HEADERS 
#  ifdef HAVE_MEMORY_H 
#   include <memory.h> 
#  endif 
# endif 
# include <string.h> 
#else 
# if HAVE_STRINGS_H 
#  include <strings.h> 
# endif 
#endif 
#if HAVE_INTTYPES_H 
# include <inttypes.h> 
#else 
# if HAVE_STDINT_H 
#  include <stdint.h> 
# endif 
#endif 
#if HAVE_UNISTD_H 
# include <unistd.h> 
#endif 
#if HAVE_MATH_H 
# include <math.h> 
#endif 
#if HAVE_FFTW3_H 
# include <fftw3.h> 
#endif 
#if HAVE_TIFF_H 
# include <tiff.h> 
#endif 
#if HAVE_TIFFIO_H 
# include <tiffio.h> 
#endif 
 
/*  
 *   Make global definitions  
 *  ****************************************************** 
*/ 
 
#define OSIV_PACKAGE_NAME "osiv" 
 
#define OSIV_VER_FIRST 0 
#define OSIV_VER_SECOND 1 
#define OSIV_VER_THIRD 1 
 
#define OSIV_TRUE ((int)1) 
#define OSIV_FALSE ((int)0) 
 
#ifndef OSIV_IM_BITS 
# define OSIV_IM_BITS 8 
#endif 
 
#ifndef OSIV_IOTAG_BYTES 
# define OSIV_IOTAG_BYTES 2 
#endif 
#ifndef OSIV_IOINT_BYTES 
# define OSIV_IOINT_BYTES 4 
#endif 
#ifndef OSIV_IOREAL_BYTES 
# define OSIV_IOREAL_BYTES 4 
#endif 
 
#ifndef OSIV_ALG_BITS 
# define OSIV_ALG_BITS 8 
#endif 
#ifndef OSIV_DISP_BITS 
# define OSIV_DISP_BITS 16 
#endif 
#ifndef OSIV_WINDOW_BITS 
# define OSIV_WINDOW_BITS 16 
#endif 
#ifndef OSIV_FRAME_BITS 
# define OSIV_FRAME_BITS 24 
#endif 
#ifndef OSIV_MOVIE_BITS 
# define OSIV_MOVIE_BITS 16 
#endif 
 
#define OSIV_IMCP_MAX (~(osiv_imcp)0) 
#define OSIV_IOINT_BLANK ((osiv_ioint)INT_MIN) 
 
#ifndef OSIV_STR_LEN 
# define OSIV_STR_LEN 1023 
#endif 
 
#define OSIV_TAG_ISBASE 0x0080      /* 0000 0000 1000 0000 */ 
 
#define OSIV_TAG_HEADER     0x0000   /* 0000 0000 0000 0000 */ 
#define OSIV_TAG_BODY       0x0001   /* 0000 0000 0000 0001 */ 
#define OSIV_TAG_ARG_INT    0x0082   /* 0000 0000 1000 0010 */ 
#define OSIV_TAG_ARG_STR    0x0083   /* 0000 0000 1000 0011 */  
#define OSIV_TAG_FRAME      0x0004   /* 0000 0000 0000 0100 */ 
#define OSIV_TAG_FRAME_NUM  0x0085   /* 0000 0000 1000 0101 */  
#define OSIV_TAG_VECTORS    0x0086   /* 0000 0000 1000 0110 */ 
#define OSIV_TAG_BOX        0x0007   /* 0000 0000 0000 0111 */ 
#define OSIV_TAG_BOX_NUM    0x0088   /* 0000 0000 1000 1000 */ 
#define OSIV_TAG_MAP        0x0089   /* 0000 0000 1000 1001 */ 
 
#define OSIV_GAT_HEADER    (~(osiv_iotag)OSIV_TAG_HEADER) 
#define OSIV_GAT_BODY      (~(osiv_iotag)OSIV_TAG_BODY) 
#define OSIV_GAT_ARG_INT   (~(osiv_iotag)OSIV_TAG_ARG_INT) 
#define OSIV_GAT_ARG_STR   (~(osiv_iotag)OSIV_TAG_ARG_STR) 
#define OSIV_GAT_FRAME     (~(osiv_iotag)OSIV_TAG_FRAME) 
#define OSIV_GAT_FRAME_NUM (~(osiv_iotag)OSIV_TAG_FRAME_NUM) 
#define OSIV_GAT_VECTORS   (~(osiv_iotag)OSIV_TAG_VECTORS) 
#define OSIV_GAT_BOX       (~(osiv_iotag)OSIV_TAG_BOX) 
#define OSIV_GAT_BOX_NUM   (~(osiv_iotag)OSIV_TAG_BOX_NUM) 
#define OSIV_GAT_MAP       (~(osiv_iotag)OSIV_TAG_MAP) 
 
#define OSIV_KEY_MAX 26 
 
#define OSIV_KEY_ALG ((osiv_ioint)0) 
#define OSIV_KEY_PREPROC ((osiv_ioint)1) 
#define OSIV_KEY_INTERP ((osiv_ioint)2) 
#define OSIV_KEY_XSIZE ((osiv_ioint)3) 
#define OSIV_KEY_YSIZE ((osiv_ioint)4) 
#define OSIV_KEY_XSTART ((osiv_ioint)5) 
#define OSIV_KEY_YSTART ((osiv_ioint)6) 
#define OSIV_KEY_XOFFSET ((osiv_ioint)7) 
#define OSIV_KEY_YOFFSET ((osiv_ioint)8) 
#define OSIV_KEY_XVECTSPACE ((osiv_ioint)9) 
#define OSIV_KEY_YVECTSPACE ((osiv_ioint)10) 
#define OSIV_KEY_XNUMVECTS ((osiv_ioint)11) 
#define OSIV_KEY_YNUMVECTS ((osiv_ioint)12) 
#define OSIV_KEY_XWINSIZE ((osiv_ioint)13) 
#define OSIV_KEY_YWINSIZE ((osiv_ioint)14) 
#define OSIV_KEY_XMAXDISP ((osiv_ioint)15) 
#define OSIV_KEY_YMAXDISP ((osiv_ioint)16) 
#define OSIV_KEY_XWINLGSIZE ((osiv_ioint)17) 
#define OSIV_KEY_YWINLGSIZE ((osiv_ioint)18) 
#define OSIV_KEY_FRMSTART ((osiv_ioint)19) 
#define OSIV_KEY_FRMFINISH ((osiv_ioint)20) 
#define OSIV_KEY_FRMSKIP ((osiv_ioint)21) 
#define OSIV_KEY_FRMSET ((osiv_ioint)22) 
#define OSIV_KEY_NUMITER ((osiv_ioint)23) 
#define OSIV_KEY_WRITEMAP ((osiv_ioint)24) 
#define OSIV_KEY_MOVIEFN ((osiv_ioint)25) 
#define OSIV_KEY_VECTORSFN ((osiv_ioint)26) 
 
/* Copy this into code when needed */  
/*  
static const char*  osiv_key_convert[] =  
  {  
    "Alg",  
    "PreProc",  
    "Interp",  
    "XSize",  
    "YSize",  
    "XStart",  
    "YStart",  
    "XOffset",  
    "YOffset",  
    "XVectSpace",  
    "YVectSpace",  
    "XNumVects",  
    "YNumVects",  
    "XWinSize",  
    "YWinSize",  
    "XMaxDisp",  
    "YMaxDisp",  
    "XWinLgSize",  
    "YWinLgSize",  
    "FrmStart",  
    "FrmFinish",  
    "FrmSkip",  
    "FrmSet",  
    "NumIter",  
    "WriteMap",  
    "MovieFN",  
    "VectorsFN"  
  };  
*/  
 
 
/*  
 *   Make type defintions 
 *  ****************************************************** 
*/ 
 
/* Set image type size */ 
#if OSIV_IM_BITS <= CHAR_BIT 
# define OSIV_IM_BYTES 1 
#elif OSIV_IM_BITS <= 2*CHAR_BIT 
# define OSIV_IM_BYTES 2 
#elif OSIV_IM_BITS <= 4*CHAR_BIT 
# define OSIV_IM_BYTES 4 
#elif OSIV_IM_BITS <= 8*CHAR_BIT 
# define OSIV_IM_BYTES 8 
#endif 
 
/* Search for suitable type for osiv_bool */ 
typedef int osiv_bool; 
 
/* Search for suitable type for osiv_ioint */ 
#if SIZEOF_INT8_T == OSIV_IOINT_BYTES 
typedef int8_t osiv_ioint; 
#elif SIZEOF_INT16_T == OSIV_IOINT_BYTES 
typedef int16_t osiv_ioint; 
#elif SIZEOF_INT32_T == OSIV_IOINT_BYTES 
typedef int32_t osiv_ioint; 
#elif SIZEOF_INT64_T == OSIV_IOINT_BYTES 
typedef int64_t osiv_ioint; 
#elif SIZEOF_SIGNED_CHAR == OSIV_IOINT_BYTES 
typedef signed char osiv_ioint; 
#elif SIZEOF_SHORT_INT == OSIV_IOINT_BYTES 
typedef short int osiv_ioint; 
#elif SIZEOF_INT == OSIV_IOINT_BYTES 
typedef int osiv_ioint; 
#elif SIZEOF_LONG_INT == OSIV_IOINT_BYTES 
typedef long int osiv_ioint; 
#elif SIZEOF_LONG_LONG_INT == OSIV_IOINT_BYTES 
typedef long long int osiv_ioint; 
#else 
# error "unable to find type for osiv_ioint !!!" 
#endif 
 
/* Search for suitable type for osiv_ioreal */ 
#if SIZEOF_FLOAT == OSIV_IOREAL_BYTES 
typedef float osiv_ioreal; 
#elif SIZEOF_DOUBLE == OSIV_IOREAL_BYTES 
typedef double osiv_ioreal; 
#elif SIZEOF_LONG_DOUBLE == OSIV_IOREAL_BYTES 
typedef long double osiv_ioreal; 
#else 
# error "unable to find type for osiv_ioreal !!!" 
#endif 
 
/* Search for suitable type for osiv_iotag */ 
#if SIZEOF_UINT8_T == OSIV_IOTAG_BYTES 
typedef uint8_t osiv_iotag; 
#elif SIZEOF_UINT16_T == OSIV_IOTAG_BYTES 
typedef uint16_t osiv_iotag; 
#elif SIZEOF_UINT32_T == OSIV_IOTAG_BYTES 
typedef uint32_t osiv_iotag; 
#elif SIZEOF_UINT64_T == OSIV_IOTAG_BYTES 
typedef uint64_t osiv_iotag; 
#elif SIZEOF_UNSIGNED_CHAR == OSIV_IOTAG_BYTES 
typedef unsigned char osiv_iotag; 
#elif SIZEOF_UNSIGNED_SHORT_INT == OSIV_IOTAG_BYTES 
typedef unsigned short int osiv_iotag; 
#elif SIZEOF_UNSIGNED_INT == OSIV_IOTAG_BYTES 
typedef unsigned int osiv_iotag; 
#elif SIZEOF_UNSIGNED_LONG_INT == OSIV_IOTAG_BYTES 
typedef unsigned long int osiv_iotag; 
#elif SIZEOF_UNSIGNED_LONG_LONG_INT == OSIV_IOTAG_BYTES 
typedef unsigned long long int osiv_iotag; 
#else 
# error "unable to find type for osiv_iotag !!!" 
#endif 
 
/* Search for suitable type for osiv_inta */ 
#if CHAR_BIT*SIZEOF_INT_FAST8_T > OSIV_ALG_BITS+1 
typedef int_fast8_t osiv_inta; 
#elif CHAR_BIT*SIZEOF_INT_FAST16_T > OSIV_ALG_BITS+1 
typedef int_fast16_t osiv_inta; 
#elif CHAR_BIT*SIZEOF_INT_FAST32_T > OSIV_ALG_BITS+1 
typedef int_fast32_t osiv_inta; 
#elif CHAR_BIT*SIZEOF_INT_FAST64_T > OSIV_ALG_BITS+1 
typedef int_fast64_t osiv_inta; 
#elif CHAR_BIT*SIZEOF_SHORT_INT > OSIV_ALG_BITS+1 
typedef short int osiv_inta; 
#elif CHAR_BIT*SIZEOF_INT > OSIV_ALG_BITS+1 
typedef int osiv_inta; 
#elif CHAR_BIT*SIZEOF_LONG_INT > OSIV_ALG_BITS+1 
typedef long int osiv_inta; 
#elif CHAR_BIT*SIZEOF_LONG_LONG_INT > OSIV_ALG_BITS+1 
typedef long long int osiv_inta; 
#else 
# error "unable to find type for osiv_inta !!!" 
#endif 
 
/* Search for suitable type for osiv_intd */ 
#if CHAR_BIT*SIZEOF_INT_FAST8_T > OSIV_DISP_BITS+1 
typedef int_fast8_t osiv_intd; 
#elif CHAR_BIT*SIZEOF_INT_FAST16_T > OSIV_DISP_BITS+1 
typedef int_fast16_t osiv_intd; 
#elif CHAR_BIT*SIZEOF_INT_FAST32_T > OSIV_DISP_BITS+1 
typedef int_fast32_t osiv_intd; 
#elif CHAR_BIT*SIZEOF_INT_FAST64_T > OSIV_DISP_BITS+1 
typedef int_fast64_t osiv_intd; 
#elif CHAR_BIT*SIZEOF_SHORT_INT > OSIV_DISP_BITS+1 
typedef short int osiv_intd; 
#elif CHAR_BIT*SIZEOF_INT > OSIV_DISP_BITS+1 
typedef int osiv_intd; 
#elif CHAR_BIT*SIZEOF_LONG_INT > OSIV_DISP_BITS+1 
typedef long int osiv_intd; 
#elif CHAR_BIT*SIZEOF_LONG_LONG_INT > OSIV_DISP_BITS+1 
typedef long long int osiv_intd; 
#else 
# error "unable to find type for osiv_intd !!!" 
#endif 
 
/* Search for suitable type for osiv_intw */ 
#if CHAR_BIT*SIZEOF_INT_FAST8_T > OSIV_WINDOW_BITS+1 
typedef int_fast8_t osiv_intw; 
#elif CHAR_BIT*SIZEOF_INT_FAST16_T > OSIV_WINDOW_BITS+1 
typedef int_fast16_t osiv_intw; 
#elif CHAR_BIT*SIZEOF_INT_FAST32_T > OSIV_WINDOW_BITS+1 
typedef int_fast32_t osiv_intw; 
#elif CHAR_BIT*SIZEOF_INT_FAST64_T > OSIV_WINDOW_BITS+1 
typedef int_fast64_t osiv_intw; 
#elif CHAR_BIT*SIZEOF_SHORT_INT > OSIV_WINDOW_BITS+1 
typedef short int osiv_intw; 
#elif CHAR_BIT*SIZEOF_INT > OSIV_WINDOW_BITS+1 
typedef int osiv_intw; 
#elif CHAR_BIT*SIZEOF_LONG_INT > OSIV_WINDOW_BITS+1 
typedef long int osiv_intw; 
#elif CHAR_BIT*SIZEOF_LONG_LONG_INT > OSIV_WINDOW_BITS+1 
typedef long long int osiv_intw; 
#else 
# error "unable to find type for osiv_intw !!!" 
#endif 
 
/* Search for suitable type for osiv_intf */ 
#if CHAR_BIT*SIZEOF_INT_FAST8_T > OSIV_FRAME_BITS+1 
typedef int_fast8_t osiv_intf; 
#elif CHAR_BIT*SIZEOF_INT_FAST16_T > OSIV_FRAME_BITS+1 
typedef int_fast16_t osiv_intf; 
#elif CHAR_BIT*SIZEOF_INT_FAST32_T > OSIV_FRAME_BITS+1 
typedef int_fast32_t osiv_intf; 
#elif CHAR_BIT*SIZEOF_INT_FAST64_T > OSIV_FRAME_BITS+1 
typedef int_fast64_t osiv_intf; 
#elif CHAR_BIT*SIZEOF_SHORT_INT > OSIV_FRAME_BITS+1 
typedef short int osiv_intf; 
#elif CHAR_BIT*SIZEOF_INT > OSIV_FRAME_BITS+1 
typedef int osiv_intf; 
#elif CHAR_BIT*SIZEOF_LONG_INT > OSIV_FRAME_BITS+1 
typedef long int osiv_intf; 
#elif CHAR_BIT*SIZEOF_LONG_LONG_INT > OSIV_FRAME_BITS+1 
typedef long long int osiv_intf; 
#else 
# error "unable to find type for osiv_intf !!!" 
#endif 
 
/* Search for suitable type for osiv_intm */ 
#if CHAR_BIT*SIZEOF_INT_FAST8_T > OSIV_MOVIE_BITS+1 
typedef int_fast8_t osiv_intm; 
#elif CHAR_BIT*SIZEOF_INT_FAST16_T > OSIV_MOVIE_BITS+1 
typedef int_fast16_t osiv_intm; 
#elif CHAR_BIT*SIZEOF_INT_FAST32_T > OSIV_MOVIE_BITS+1 
typedef int_fast32_t osiv_intm; 
#elif CHAR_BIT*SIZEOF_INT_FAST64_T > OSIV_MOVIE_BITS+1 
typedef int_fast64_t osiv_intm; 
#elif CHAR_BIT*SIZEOF_SHORT_INT > OSIV_MOVIE_BITS+1 
typedef short int osiv_intm; 
#elif CHAR_BIT*SIZEOF_INT > OSIV_MOVIE_BITS+1 
typedef int osiv_intm; 
#elif CHAR_BIT*SIZEOF_LONG_INT > OSIV_MOVIE_BITS+1 
typedef long int osiv_intm; 
#elif CHAR_BIT*SIZEOF_LONG_LONG_INT > OSIV_MOVIE_BITS+1 
typedef long long int osiv_intm; 
#else 
# error "unable to find type for osiv_intm !!!" 
#endif 
 
/* Search for suitable type for osiv_int8 */ 
#if CHAR_BIT*SIZEOF_INT8_T == 8 
typedef int8_t osiv_int8; 
#elif CHAR_BIT*SIZEOF_INT16_T == 8 
typedef int16_t osiv_int8; 
#elif CHAR_BIT*SIZEOF_INT32_T == 8 
typedef int32_t osiv_int8; 
#elif CHAR_BIT*SIZEOF_INT64_T == 8 
typedef int64_t osiv_int8; 
#elif CHAR_BIT*SIZEOF_SHORT_INT == 8 
typedef short int osiv_int8; 
#elif CHAR_BIT*SIZEOF_INT == 8 
typedef int osiv_int8; 
#elif CHAR_BIT*SIZEOF_LONG_INT == 8 
typedef long int osiv_int8; 
#elif CHAR_BIT*SIZEOF_LONG_LONG_INT == 8 
typedef long long int osiv_int8; 
#else 
# error "unable to find type for osiv_int8 !!!" 
#endif 
 
/* Search for suitable type for osiv_int16 */ 
#if CHAR_BIT*SIZEOF_INT8_T == 16 
typedef int8_t osiv_int16; 
#elif CHAR_BIT*SIZEOF_INT16_T == 16 
typedef int16_t osiv_int16; 
#elif CHAR_BIT*SIZEOF_INT32_T == 16 
typedef int32_t osiv_int16; 
#elif CHAR_BIT*SIZEOF_INT64_T == 16 
typedef int64_t osiv_int16; 
#elif CHAR_BIT*SIZEOF_SHORT_INT == 16 
typedef short int osiv_int16; 
#elif CHAR_BIT*SIZEOF_INT == 16 
typedef int osiv_int16; 
#elif CHAR_BIT*SIZEOF_LONG_INT == 16 
typedef long int osiv_int16; 
#elif CHAR_BIT*SIZEOF_LONG_LONG_INT == 16 
typedef long long int osiv_int16; 
#else 
# error "unable to find type for osiv_int16 !!!" 
#endif 
 
/* Search for suitable type for osiv_int32 */ 
#if CHAR_BIT*SIZEOF_INT8_T == 32 
typedef int8_t osiv_int32; 
#elif CHAR_BIT*SIZEOF_INT16_T == 32 
typedef int16_t osiv_int32; 
#elif CHAR_BIT*SIZEOF_INT32_T == 32 
typedef int32_t osiv_int32; 
#elif CHAR_BIT*SIZEOF_INT64_T == 32 
typedef int64_t osiv_int32; 
#elif CHAR_BIT*SIZEOF_SHORT_INT == 32 
typedef short int osiv_int32; 
#elif CHAR_BIT*SIZEOF_INT == 32 
typedef int osiv_int32; 
#elif CHAR_BIT*SIZEOF_LONG_INT == 32 
typedef long int osiv_int32; 
#elif CHAR_BIT*SIZEOF_LONG_LONG_INT == 32 
typedef long long int osiv_int32; 
#else 
# error "unable to find type for osiv_int32 !!!" 
#endif 
 
/* Search for suitable type for osiv_int64 */ 
#if CHAR_BIT*SIZEOF_INT8_T == 64 
typedef int8_t osiv_int64; 
#elif CHAR_BIT*SIZEOF_INT16_T == 64 
typedef int16_t osiv_int64; 
#elif CHAR_BIT*SIZEOF_INT32_T == 64 
typedef int32_t osiv_int64; 
#elif CHAR_BIT*SIZEOF_INT64_T == 64 
typedef int64_t osiv_int64; 
#elif CHAR_BIT*SIZEOF_SHORT_INT == 64 
typedef short int osiv_int64; 
#elif CHAR_BIT*SIZEOF_INT == 64 
typedef int osiv_int64; 
#elif CHAR_BIT*SIZEOF_LONG_INT == 64 
typedef long int osiv_int64; 
#elif CHAR_BIT*SIZEOF_LONG_LONG_INT == 64 
typedef long long int osiv_int64; 
#else 
# error "unable to find type for osiv_int64 !!!" 
#endif 
 
/* Search for suitable type for osiv_uint8 */ 
#if CHAR_BIT*SIZEOF_UINT8_T == 8 
typedef uint8_t osiv_uint8; 
#elif CHAR_BIT*SIZEOF_UINT16_T == 8 
typedef uint16_t osiv_uint8; 
#elif CHAR_BIT*SIZEOF_UINT32_T == 8 
typedef uint32_t osiv_uint8; 
#elif CHAR_BIT*SIZEOF_UINT64_T == 8 
typedef uint64_t osiv_uint8; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_SHORT_INT == 8 
typedef unsigned short int osiv_uint8; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_INT == 8 
typedef unsigned int osiv_uint8; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_INT == 8 
typedef unsigned long int osiv_uint8; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_LONG_INT == 8 
typedef unsigned long long int osiv_uint8; 
#else 
# error "unable to find type for osiv_uint8 !!!" 
#endif 
 
/* Search for suitable type for osiv_uint16 */ 
#if CHAR_BIT*SIZEOF_UINT8_T == 16 
typedef uint8_t osiv_uint16; 
#elif CHAR_BIT*SIZEOF_UINT16_T == 16 
typedef uint16_t osiv_uint16; 
#elif CHAR_BIT*SIZEOF_UINT32_T == 16 
typedef uint32_t osiv_uint16; 
#elif CHAR_BIT*SIZEOF_UINT64_T == 16 
typedef uint64_t osiv_uint16; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_SHORT_INT == 16 
typedef unsigned short int osiv_uint16; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_INT == 16 
typedef unsigned int osiv_uint16; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_INT == 16 
typedef unsigned long int osiv_uint16; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_LONG_INT == 16 
typedef unsigned long long int osiv_uint16; 
#else 
# error "unable to find type for osiv_uint16 !!!" 
#endif 
 
/* Search for suitable type for osiv_uint32 */ 
#if CHAR_BIT*SIZEOF_UINT8_T == 32 
typedef uint8_t osiv_uint32; 
#elif CHAR_BIT*SIZEOF_UINT16_T == 32 
typedef uint16_t osiv_uint32; 
#elif CHAR_BIT*SIZEOF_UINT32_T == 32 
typedef uint32_t osiv_uint32; 
#elif CHAR_BIT*SIZEOF_UINT64_T == 32 
typedef uint64_t osiv_uint32; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_SHORT_INT == 32 
typedef short int osiv_uint32; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_INT == 32 
typedef unsigned int osiv_uint32; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_INT == 32 
typedef unsigned long int osiv_uint32; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_LONG_INT == 32 
typedef unsigned long long int osiv_uint32; 
#else 
# error "unable to find type for osiv_uint32 !!!" 
#endif 
 
/* Search for suitable type for osiv_uint64 */ 
#if CHAR_BIT*SIZEOF_UINT8_T == 64 
typedef uint8_t osiv_uint64; 
#elif CHAR_BIT*SIZEOF_UINT16_T == 64 
typedef uint16_t osiv_uint64; 
#elif CHAR_BIT*SIZEOF_UINT32_T == 64 
typedef uint32_t osiv_uint64; 
#elif CHAR_BIT*SIZEOF_UINT64_T == 64 
typedef uint64_t osiv_uint64; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_SHORT_INT == 64 
typedef unsigned short int osiv_uint64; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_INT == 64 
typedef unsigned int osiv_uint64; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_INT == 64 
typedef unsigned long int osiv_uint64; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_LONG_INT == 64 
typedef unsigned long long int osiv_uint64; 
#else 
# error "unable to find type for osiv_uint64 !!!" 
#endif 
 
/* Search for suitable type for osiv_imcp */ 
#if (OSIV_IM_BYTES == 1) && defined(__CHAR_UNSIGNED__) 
typedef char osiv_imcp; 
#elif SIZEOF_UNSIGNED_CHAR == OSIV_IM_BYTES 
typedef unsigned char osiv_imcp; 
#elif SIZEOF_UNSIGNED_SHORT_INT == OSIV_IM_BYTES 
typedef unsigned short int osiv_imcp; 
#elif SIZEOF_UNSIGNED_INT == OSIV_IM_BYTES 
typedef unsigned int osiv_imcp; 
#elif SIZEOF_UNSIGNED_LONG_INT == OSIV_IM_BYTES 
typedef unsigned long int osiv_imcp; 
#elif SIZEOF_UNSIGNED_LONG_LONG_INT == OSIV_IM_BYTES 
typedef unsigned long long int osiv_imcp; 
#elif SIZEOF_UINT8_T == OSIV_IM_BYTES 
typedef uint8_t osiv_imcp; 
#elif SIZEOF_UINT16_T == OSIV_IM_BYTES 
typedef uint16_t osiv_imcp; 
#elif SIZEOF_UINT32_T == OSIV_IM_BYTES 
typedef uint32_t osiv_imcp; 
#elif SIZEOF_UINT64_T == OSIV_IM_BYTES 
typedef uint64_t osiv_imcp; 
#else 
# error "unable to find type for osiv_imcp !!!" 
#endif 
 
/* Search for suitable type for osiv_imsp */ 
#if CHAR_BIT*SIZEOF_INT_FAST8_T >= OSIV_IM_BITS+1 
typedef int_fast8_t osiv_imsp; 
#elif CHAR_BIT*SIZEOF_INT_FAST16_T >= OSIV_IM_BITS+1 
typedef int_fast16_t osiv_imsp; 
#elif CHAR_BIT*SIZEOF_INT_FAST32_T >= OSIV_IM_BITS+1 
typedef int_fast32_t osiv_imsp; 
#elif CHAR_BIT*SIZEOF_INT_FAST64_T >= OSIV_IM_BITS+1 
typedef int_fast64_t osiv_imsp; 
#elif CHAR_BIT*SIZEOF_SIGNED_CHAR >= OSIV_IM_BITS+1 
typedef signed char osiv_imsp; 
#elif CHAR_BIT*SIZEOF_SHORT_INT >= OSIV_IM_BITS+1 
typedef short int osiv_imsp; 
#elif CHAR_BIT*SIZEOF_INT >= OSIV_IM_BITS+1 
typedef int osiv_imsp; 
#elif CHAR_BIT*SIZEOF_LONG_INT >= OSIV_IM_BITS+1 
typedef long int osiv_imsp; 
#elif CHAR_BIT*SIZEOF_LONG_LONG_INT >= OSIV_IM_BITS+1 
typedef long long int osiv_imsp; 
#else 
# error "unable to find type for osiv_imsp !!!" 
#endif 
 
/* Search for suitable type for osiv_immp */ 
#if CHAR_BIT*SIZEOF_UINT_FAST8_T >= 2*OSIV_IM_BITS 
typedef uint_fast8_t osiv_immp; 
#elif CHAR_BIT*SIZEOF_UINT_FAST16_T >= 2*OSIV_IM_BITS 
typedef uint_fast16_t osiv_immp; 
#elif CHAR_BIT*SIZEOF_UINT_FAST32_T >= 2*OSIV_IM_BITS 
typedef uint_fast32_t osiv_immp; 
#elif CHAR_BIT*SIZEOF_UINT_FAST64_T >= 2*OSIV_IM_BITS 
typedef uint_fast64_t osiv_immp; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_CHAR >= 2*OSIV_IM_BITS 
typedef unsigned char osiv_immp; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_SHORT_INT >= 2*OSIV_IM_BITS 
typedef unsigned short int osiv_immp; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_INT >= 2*OSIV_IM_BITS 
typedef unsigned int osiv_immp; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_INT >= 2*OSIV_IM_BITS 
typedef unsigned long int osiv_immp; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_LONG_INT >= 2*OSIV_IM_BITS 
typedef unsigned long long int osiv_immp; 
#else 
# error "unable to find type for osiv_immp !!!" 
#endif 
 
/* Search for suitable type for osiv_imcw */ 
#if CHAR_BIT*SIZEOF_UINT_FAST8_T >= OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef uint_fast8_t osiv_imcw; 
#elif CHAR_BIT*SIZEOF_UINT_FAST16_T >= OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef uint_fast16_t osiv_imcw; 
#elif CHAR_BIT*SIZEOF_UINT_FAST32_T >= OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef uint_fast32_t osiv_imcw; 
#elif CHAR_BIT*SIZEOF_UINT_FAST64_T >= OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef uint_fast64_t osiv_imcw; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_CHAR >= OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef unsigned char osiv_imcw; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_SHORT_INT >= OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef unsigned short int osiv_imcw; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_INT >= OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef unsigned int osiv_imcw; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_INT >= OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef unsigned long int osiv_imcw; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_LONG_INT >= OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef unsigned long long int osiv_imcw; 
#else 
# error "unable to find type for osiv_imcw !!!" 
#endif 
 
/* Search for suitable type for osiv_imsw */ 
#if CHAR_BIT*SIZEOF_INT_FAST8_T >= OSIV_IM_BITS+OSIV_WINDOW_BITS+1 
typedef int_fast8_t osiv_imsw; 
#elif CHAR_BIT*SIZEOF_INT_FAST16_T >= OSIV_IM_BITS+OSIV_WINDOW_BITS+1 
typedef int_fast16_t osiv_imsw; 
#elif CHAR_BIT*SIZEOF_INT_FAST32_T >= OSIV_IM_BITS+OSIV_WINDOW_BITS+1 
typedef int_fast32_t osiv_imsw; 
#elif CHAR_BIT*SIZEOF_INT_FAST64_T >= OSIV_IM_BITS+OSIV_WINDOW_BITS+1 
typedef int_fast64_t osiv_imsw; 
#elif CHAR_BIT*SIZEOF_SIGNED_CHAR >= OSIV_IM_BITS+OSIV_WINDOW_BITS+1 
typedef signed char osiv_imsw; 
#elif CHAR_BIT*SIZEOF_SHORT_INT >= OSIV_IM_BITS+OSIV_WINDOW_BITS+1 
typedef short int osiv_imsw; 
#elif CHAR_BIT*SIZEOF_INT >= OSIV_IM_BITS+OSIV_WINDOW_BITS+1 
typedef int osiv_imsw; 
#elif CHAR_BIT*SIZEOF_LONG_INT >= OSIV_IM_BITS+OSIV_WINDOW_BITS+1 
typedef long int osiv_imsw; 
#elif CHAR_BIT*SIZEOF_LONG_LONG_INT >= OSIV_IM_BITS+OSIV_WINDOW_BITS+1 
typedef long long int osiv_imsw; 
#else 
# error "unable to find type for osiv_imsw !!!" 
#endif 
 
/* Search for suitable type for osiv_immw */ 
#if CHAR_BIT*SIZEOF_UINT_FAST8_T >= 2*OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef uint_fast8_t osiv_immw; 
#elif CHAR_BIT*SIZEOF_UINT_FAST16_T >= 2*OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef uint_fast16_t osiv_immw; 
#elif CHAR_BIT*SIZEOF_UINT_FAST32_T >= 2*OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef uint_fast32_t osiv_immw; 
#elif CHAR_BIT*SIZEOF_UINT_FAST64_T >= 2*OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef uint_fast64_t osiv_immw; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_CHAR >= 2*OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef unsigned char osiv_immw; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_SHORT_INT >= 2*OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef unsigned short int osiv_immw; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_INT >= 2*OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef unsigned int osiv_immw; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_INT >= 2*OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef unsigned long int osiv_immw; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_LONG_INT >= 2*OSIV_IM_BITS+OSIV_WINDOW_BITS 
typedef unsigned long long int osiv_immw; 
#else 
# error "unable to find type for osiv_immw !!!" 
#endif 
 
/* Search for suitable type for osiv_imcf */ 
#if CHAR_BIT*SIZEOF_UINT_FAST8_T >= OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef uint_fast8_t osiv_imcf; 
#elif CHAR_BIT*SIZEOF_UINT_FAST16_T >= OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef uint_fast16_t osiv_imcf; 
#elif CHAR_BIT*SIZEOF_UINT_FAST32_T >= OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef uint_fast32_t osiv_imcf; 
#elif CHAR_BIT*SIZEOF_UINT_FAST64_T >= OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef uint_fast64_t osiv_imcf; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_CHAR >= OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef unsigned char osiv_imcf; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_SHORT_INT >= OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef unsigned short int osiv_imcf; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_INT >= OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef unsigned int osiv_imcf; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_INT >= OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef unsigned long int osiv_imcf; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_LONG_INT >= OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef unsigned long long int osiv_imcf; 
#else 
# error "unable to find type for osiv_imcf !!!" 
#endif 
 
/* Search for suitable type for osiv_imsf */ 
#if CHAR_BIT*SIZEOF_INT_FAST8_T >= OSIV_IM_BITS+OSIV_FRAME_BITS+1 
typedef int_fast8_t osiv_imsf; 
#elif CHAR_BIT*SIZEOF_INT_FAST16_T >= OSIV_IM_BITS+OSIV_FRAME_BITS+1 
typedef int_fast16_t osiv_imsf; 
#elif CHAR_BIT*SIZEOF_INT_FAST32_T >= OSIV_IM_BITS+OSIV_FRAME_BITS+1 
typedef int_fast32_t osiv_imsf; 
#elif CHAR_BIT*SIZEOF_INT_FAST64_T >= OSIV_IM_BITS+OSIV_FRAME_BITS+1 
typedef int_fast64_t osiv_imsf; 
#elif CHAR_BIT*SIZEOF_SIGNED_CHAR >= OSIV_IM_BITS+OSIV_FRAME_BITS+1 
typedef signed char osiv_imsf; 
#elif CHAR_BIT*SIZEOF_SHORT_INT >= OSIV_IM_BITS+OSIV_FRAME_BITS+1 
typedef short int osiv_imsf; 
#elif CHAR_BIT*SIZEOF_INT >= OSIV_IM_BITS+OSIV_FRAME_BITS+1 
typedef int osiv_imsf; 
#elif CHAR_BIT*SIZEOF_LONG_INT >= OSIV_IM_BITS+OSIV_FRAME_BITS+1 
typedef long int osiv_imsf; 
#elif CHAR_BIT*SIZEOF_LONG_LONG_INT >= OSIV_IM_BITS+OSIV_FRAME_BITS+1 
typedef long long int osiv_imsf; 
#else 
# error "unable to find type for osiv_imsf !!!" 
#endif 
 
/* Search for suitable type for osiv_immf */ 
#if CHAR_BIT*SIZEOF_UINT_FAST8_T >= 2*OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef uint_fast8_t osiv_immf; 
#elif CHAR_BIT*SIZEOF_UINT_FAST16_T >= 2*OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef uint_fast16_t osiv_immf; 
#elif CHAR_BIT*SIZEOF_UINT_FAST32_T >= 2*OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef uint_fast32_t osiv_immf; 
#elif CHAR_BIT*SIZEOF_UINT_FAST64_T >= 2*OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef uint_fast64_t osiv_immf; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_CHAR >= 2*OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef unsigned char osiv_immf; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_SHORT_INT >= 2*OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef unsigned short int osiv_immf; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_INT >= 2*OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef unsigned int osiv_immf; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_INT >= 2*OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef unsigned long int osiv_immf; 
#elif CHAR_BIT*SIZEOF_UNSIGNED_LONG_LONG_INT >= 2*OSIV_IM_BITS+OSIV_FRAME_BITS 
typedef unsigned long long int osiv_immf; 
#else 
# error "unable to find type for osiv_immf !!!" 
#endif 
 
/* Search for type for osiv_real */ 
#if OSIV_USE_SINGLE 
typedef float osiv_real; 
#else 
typedef double osiv_real; 
#endif 
 
/* Make fftw type */ 
typedef double fftw_real; 
 
/* Put together movie types */ 
enum osiv_movie_type 
{ 
  OSIV_MOVIE_STREAM, 
  OSIV_MOVIE_FILE, 
  OSIV_MOVIE_TIFF_MERGE, 
  OSIV_MOVIE_TIFF_SPLIT 
}; 
 
/* Construct types for parameters */ 
typedef struct 
{	 
  FILE* VectorsFile; 
   
  void* MovieData; 
  void* PreProcData; 
  osiv_inta MovieType; 
 
  osiv_inta PreProc; 
  osiv_inta Interp; 
  osiv_intf XSize; 
  osiv_intf YSize; 
  osiv_intf XStart; 
  osiv_intf YStart; 
  osiv_intf XOffset; 
  osiv_intf YOffset; 
  osiv_intf XVectSpace; 
  osiv_intf YVectSpace; 
  osiv_intf XNumVects; 
  osiv_intf YNumVects; 
  osiv_intw XWinSize; 
  osiv_intw YWinSize; 
  osiv_intd XMaxDisp; 
  osiv_intd YMaxDisp; 
  osiv_intw XWinLgSize; 
  osiv_intw YWinLgSize; 
  osiv_intm FrmStart; 
  osiv_intm FrmFinish; 
  osiv_intm FrmSkip; 
  osiv_intm FrmSet; 
  osiv_inta NumIter; 
  osiv_intf NumThreads; 
  osiv_inta WriteMap; 
 
  osiv_intd XMapSize; 
  osiv_intd YMapSize; 
  osiv_intd MapLength; 
 
  osiv_intw XFFTSize; 
  osiv_intw YFFTSize; 
  osiv_intw FFTLength; 
 
  osiv_intw XFFTLgSize; 
  osiv_intw YFFTLgSize; 
  osiv_intw FFTLgLength; 
 
  osiv_intw WinLength; 
  osiv_intf ImageLength; 
  osiv_intf NumVects; 
 
  osiv_inta SwapReal; 
} osiv_param; 
 
typedef struct 
{ 
  osiv_ioint Alg; 
  osiv_ioint PreProc; 
  osiv_ioint Interp; 
  osiv_ioint XSize; 
  osiv_ioint YSize; 
  osiv_ioint XStart; 
  osiv_ioint YStart; 
  osiv_ioint XOffset; 
  osiv_ioint YOffset; 
  osiv_ioint XVectSpace; 
  osiv_ioint YVectSpace; 
  osiv_ioint XNumVects; 
  osiv_ioint YNumVects; 
  osiv_ioint XWinSize; 
  osiv_ioint YWinSize; 
  osiv_ioint XMaxDisp; 
  osiv_ioint YMaxDisp; 
  osiv_ioint XWinLgSize; 
  osiv_ioint YWinLgSize; 
  osiv_ioint FrmStart; 
  osiv_ioint FrmFinish; 
  osiv_ioint FrmSkip; 
  osiv_ioint FrmSet; 
  osiv_ioint NumIter; 
  osiv_ioint NumThreads; 
  osiv_ioint WriteMap; 
 
  char* MovieFN; 
  char* VectorsFN; 
} osiv_input; 
 
typedef struct 
{ 
  FILE* stream; 
 
  int swap_bytes; 
  int swap_bits; 
 
  size_t iotag_size; 
  size_t ioint_size; 
  size_t ioreal_size; 
 
  char ver_first; 
  char ver_second; 
  char ver_third; 
 
  long pre_len; 
} osiv_file; 
 
 
/*  
 *   Make function prototypes 
 *  ****************************************************** 
*/ 
 
/* alg1.c */ 
void OsivAlg1_CheckOpts(osiv_input *input); 
void OsivAlg1(osiv_param *p); 
 
/* alg2.c */ 
void OsivAlg2_CheckOpts(osiv_input *input); 
void OsivAlg2(osiv_param *p); 
 
/* alg3.c */ 
void OsivAlg3_CheckOpts(osiv_input *input); 
void OsivAlg3(osiv_param *p); 
 
/* alg4.c */ 
void OsivAlg4_CheckOpts(osiv_input *input); 
void OsivAlg4(osiv_param *p); 
 
/* alg5.c */ 
void OsivAlg5_CheckOpts(osiv_input *input); 
void OsivAlg5(osiv_param *p); 
 
/* alg6.c */ 
void OsivAlg6_CheckOpts(osiv_input *input); 
void OsivAlg6(osiv_param *p); 
 
/* alg7.c */ 
void OsivAlg7_CheckOpts(osiv_input *input); 
void OsivAlg7(osiv_param *p); 
 
/* alg8.c */ 
void OsivAlg8_CheckOpts(osiv_input *input); 
void OsivAlg8(osiv_param *p); 
 
/* alg9.c */ 
#ifndef OSIV_NO_THREADS 
void OsivAlg9_CheckOpts(osiv_input *input); 
void OsivAlg9(osiv_param *p); 
#endif 
 
/* extrema.c */ 
osiv_intf OsivGetMinInd_IMCP(osiv_imcp *array, osiv_intf len); 
osiv_intf OsivGetMinInd_IMSP(osiv_imsp *array, osiv_intf len); 
osiv_intf OsivGetMinInd_IMMP(osiv_immp *array, osiv_intf len); 
osiv_intw OsivGetMinInd_IMCW(osiv_imcw *array, osiv_intw len); 
osiv_intw OsivGetMinInd_IMSW(osiv_immw *array, osiv_intw len); 
osiv_intw OsivGetMinInd_IMMW(osiv_immw *array, osiv_intw len); 
osiv_intf OsivGetMinInd_IMCF(osiv_imcf *array, osiv_intf len); 
osiv_intf OsivGetMinInd_IMSF(osiv_imsf *array, osiv_intf len); 
osiv_intf OsivGetMinInd_IMMF(osiv_immf *array, osiv_intf len); 
osiv_intf OsivGetMaxInd_IMCP(osiv_imcp *array, osiv_intf len); 
osiv_intf OsivGetMaxInd_IMSP(osiv_imsp *array, osiv_intf len); 
osiv_intf OsivGetMaxInd_IMMP(osiv_immp *array, osiv_intf len); 
osiv_intw OsivGetMaxInd_IMCW(osiv_imcw *array, osiv_intw len); 
osiv_intw OsivGetMaxInd_IMSW(osiv_imsw *array, osiv_intw len); 
osiv_intw OsivGetMaxInd_IMMW(osiv_immw *array, osiv_intw len); 
osiv_intf OsivGetMaxInd_IMCF(osiv_imcf *array, osiv_intf len); 
osiv_intf OsivGetMaxInd_IMSF(osiv_imsf *array, osiv_intf len); 
osiv_intf OsivGetMaxInd_IMMF(osiv_immf *array, osiv_intf len); 
osiv_intw OsivGetMinInd_FFT(fftw_real *array, osiv_intw xlen, osiv_intw ylen,  
			    osiv_intw xdisp, osiv_intw ydisp, osiv_intw yskip); 
osiv_intw OsivGetMaxInd_FFT(fftw_real *array, osiv_intw xlen, osiv_intw ylen,  
			    osiv_intw xdisp, osiv_intw ydisp, osiv_intw yskip); 
 
/* interp.c */ 
void OsivInterpMin_IMCP(osiv_inta method, osiv_imcp *ind,  
			osiv_intf lft, osiv_intf rht,  
			osiv_intf up, osiv_intf dwn,  
			osiv_real *x, osiv_real *y); 
void OsivInterpMin_IMSP(osiv_inta method, osiv_imsp *ind,  
			osiv_intf lft, osiv_intf rht,  
			osiv_intf up, osiv_intf dwn,  
			osiv_real *x, osiv_real *y); 
void OsivInterpMin_IMMP(osiv_inta method, osiv_immp *ind,  
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
void OsivInterpMin_IMCW(osiv_inta method, osiv_imcw *ind, 
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
void OsivInterpMin_IMSW(osiv_inta method, osiv_imsw *ind, 
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
void OsivInterpMin_IMMW(osiv_inta method, osiv_immw *ind, 
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
void OsivInterpMin_IMCF(osiv_inta method, osiv_imcf *ind, 
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
void OsivInterpMin_IMSF(osiv_inta method, osiv_imsf *ind, 
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
void OsivInterpMin_IMMF(osiv_inta method, osiv_immf *ind,  
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn,  
			osiv_real *x, osiv_real *y); 
void OsivInterpMin_REAL(osiv_inta method, osiv_real *ind, 
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
void OsivInterpMin_FFTW(osiv_inta method, fftw_real *ind, 
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
void OsivInterpMax_IMCP(osiv_inta method, osiv_imcp *ind,  
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
void OsivInterpMax_IMSP(osiv_inta method, osiv_imsp *ind,  
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
void OsivInterpMax_IMMP(osiv_inta method, osiv_immp *ind, 
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
void OsivInterpMax_IMCW(osiv_inta method, osiv_imcw *ind, 
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
void OsivInterpMax_IMSW(osiv_inta method, osiv_imsw *ind, 
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn,  
			osiv_real *x, osiv_real *y); 
void OsivInterpMax_IMMW(osiv_inta method, osiv_immw *ind, 
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
void OsivInterpMax_IMCF(osiv_inta method, osiv_imcf *ind, 
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
void OsivInterpMax_IMSF(osiv_inta method, osiv_imsf *ind, 
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
void OsivInterpMax_IMMF(osiv_inta method, osiv_immf *ind, 
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, osiv_real *x, osiv_real *y); 
void OsivInterpMax_REAL(osiv_inta method, osiv_real *ind, 
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
void OsivInterpMax_FFTW(osiv_inta method, fftw_real *ind, 
			osiv_intf lft, osiv_intf rht, 
			osiv_intf up, osiv_intf dwn, 
			osiv_real *x, osiv_real *y); 
 
/* load.c */ 
osiv_file *OsivOpenIO(FILE *fp); 
osiv_file *OsivOpen(const char *fn); 
osiv_bool OsivClose(osiv_file *file); 
osiv_bool OsivRead(void *ptr, size_t size, size_t nitems, osiv_file *file); 
osiv_bool OsivRewind(osiv_file *fp); 
osiv_bool OsivTell(osiv_file *fp, long *pos); 
osiv_bool OsivSeek(osiv_file *fp, long offset, int whence); 
osiv_bool OsivReadVer(osiv_file *fp, char *ver); 
osiv_bool OsivReadTag(osiv_file *file, osiv_iotag *tag); 
osiv_bool OsivReadArgInt(osiv_file *file, osiv_ioint *key, osiv_ioint *val); 
osiv_bool OsivReadArgStr(osiv_file *file, osiv_ioint *key, char **val); 
osiv_bool OsivReadFrameNum(osiv_file *file, osiv_ioint *frame); 
osiv_bool OsivReadVectors(osiv_file *file, osiv_ioint *len, osiv_ioreal **val); 
osiv_bool OsivReadBoxNum(osiv_file *file, osiv_ioint *val); 
osiv_bool OsivReadMap(osiv_file *file, osiv_ioint *len, osiv_ioreal **map); 
osiv_bool OsivTagCmp(osiv_iotag a, osiv_iotag b); 
osiv_bool OsivScanToTag(osiv_file *file, osiv_iotag tag); 
osiv_bool OsivScanToFrame(osiv_file *file, osiv_intm num); 
osiv_bool OsivScanToBox(osiv_file *file, osiv_intf num); 
osiv_bool OsivGrabArgInt(osiv_file *file, osiv_ioint key, osiv_ioint* val); 
osiv_bool OsivGrabArgStr(osiv_file *file, osiv_ioint key, char** val); 
osiv_bool OsivGrabVectors(osiv_file *file, osiv_intm frame,  
			  osiv_ioint *vectors_len, osiv_ioreal **vectors); 
osiv_bool OsivGrabMap(osiv_file *file, osiv_intm frame, osiv_intf window,  
		      osiv_ioint *map_len, osiv_ioreal **map); 
 
/* movie.c */ 
void OsivSetupMovie(osiv_param *p, const char *fn); 
void OsivGetImage(osiv_param *p, osiv_intm frm, osiv_imcp *im); 
void OsivCloseMovie(osiv_param *p); 
 
/* osiv.c */ 
void OsivError(const char *m); 
void OsivError2(const char *m1, const char *m2);  
void *OsivCalloc(size_t nmemb, size_t size); 
void *OsivMalloc(size_t nbytes); 
 
/* parse.c */ 
void OsivParseInput(osiv_input *input, int argc, char **args); 
 
/* preproc.c */ 
void OsivSetupPreProc(osiv_param *p); 
void OsivPreProc(osiv_param *p, osiv_intm frm, osiv_imcp *image); 
void OsivClosePreProc(osiv_param *p); 
 
/* save.c */ 
osiv_bool OsivWriteTag(FILE *fp, osiv_iotag tag); 
osiv_bool OsivWriteArgInt(FILE *fp, osiv_ioint key, osiv_ioint val); 
osiv_bool OsivWriteArgStr(FILE *fp, osiv_ioint key, char *str); 
osiv_bool OsivWriteFrameNum(FILE *fp, osiv_ioint val); 
osiv_bool OsivWriteVectors(FILE *fp, osiv_ioint len, osiv_ioreal *array); 
osiv_bool OsivWriteBoxNum(FILE *fp, osiv_ioint val); 
osiv_bool OsivWriteMap(FILE *fp, osiv_ioint len, osiv_ioreal *map); 
osiv_bool OsivWriteHeader(FILE *fp, osiv_input *input); 
 
#ifndef OSIV_HAVE_RINT 
# define rint(a) (ceil(a-.5)) 
#endif 
 
#define _OSIV_H 
#endif 
