/* File: preproc.c

 * Copyright (c) 2002 James A. Strother
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
*/

#include "osiv.h"

static void OsivSetupPreProc_Method1 (osiv_param* p)
{
  osiv_intm i;
  osiv_imcp* im;
  osiv_imcp* min;

  im = (osiv_imcp*)OsivMalloc(p->ImageLength*sizeof(osiv_imcp));
  min = (osiv_imcp*)OsivMalloc(p->ImageLength*sizeof(osiv_imcp));

  OsivGetImage(p,p->FrmFinish,im);
  memcpy(min,im,p->ImageLength*sizeof(osiv_imcp));
  for (i = p->FrmStart; i < p->FrmFinish; i += p->FrmSkip)
    {
      osiv_intf j;
      
      OsivGetImage(p,i,im);
      for (j = 0; j < p->ImageLength; j++)
	{
	  if (*(im+j) < *(min+j))
	    *(min+j) = *(im+j);
	}
    }
  free(im);

  p->PreProcData = (void*)min;
}

static void OsivSetupPreProc_Method2 (osiv_param* p)
{
  osiv_intm i;
  osiv_imcp* im;
  osiv_imcp* max;

  im = (osiv_imcp*)OsivMalloc(p->ImageLength*sizeof(osiv_imcp));
  max = (osiv_imcp*)OsivMalloc(p->ImageLength*sizeof(osiv_imcp));

  OsivGetImage(p,p->FrmFinish,im);
  memcpy(max,im,p->ImageLength*sizeof(osiv_imcp));
  for (i = p->FrmStart; i < p->FrmFinish; i += p->FrmSkip)
    {
      osiv_intf j;
      
      OsivGetImage(p,i,im);
      for (j = 0; j < p->ImageLength; j++)
	{
	  if (*(im+j) > *(max+j))
	    *(max+j) = *(im+j);
	}
    }
  free(im);

  p->PreProcData = (void*)max;
}

static void OsivSetupPreProc_Method3 (osiv_param* p)
{
  osiv_intm i;
  osiv_imcp* im;
  osiv_imcp* min;
  osiv_imcp* max;

  im = (osiv_imcp*)OsivMalloc(p->ImageLength*sizeof(osiv_imcp));
  min = (osiv_imcp*)OsivMalloc(2*p->ImageLength*sizeof(osiv_imcp));
  max = min + p->ImageLength;

  OsivGetImage(p,p->FrmFinish,im);
  memcpy(min,im,p->ImageLength*sizeof(osiv_imcp));
  memcpy(max,im,p->ImageLength*sizeof(osiv_imcp));
  for (i = p->FrmStart; i < p->FrmFinish; i += p->FrmSkip)
    {
      osiv_intf j;
      
      OsivGetImage(p,i,im);
      for (j = 0; j < p->ImageLength; j++)
	{
	  if (*(im+j) < *(min+j))
	    *(min+j) = *(im+j);
	  if (*(im+j) > *(max+j))
	    *(max+j) = *(im+j);
	}
    }
  free(im);

  p->PreProcData = (void*)min;
}

static void OsivSetupPreProc_Method4 (osiv_param* p)
{
  osiv_intm i;
  osiv_intf j;
  osiv_imcp* im;
  osiv_real* ex;
  osiv_real* ex2;

  im = (osiv_imcp*)OsivMalloc(p->ImageLength*sizeof(osiv_imcp));
  ex = (osiv_real*)OsivCalloc(2*p->ImageLength,sizeof(osiv_real));
  ex2 = ex + p->ImageLength;

  for (i = p->FrmStart; i <= p->FrmFinish; i += p->FrmSkip)
    {
      OsivGetImage(p,i,im);
      for (j = 0; j < p->ImageLength; j++)
	{
	  *(ex+j) += *(im+j);
	  *(ex2+j) += *(im+j)**(im+j);
	}
    }

  i = (p->FrmFinish-p->FrmStart)/p->FrmSkip+1;
  for (j = 0; j < p->ImageLength; j++)
    {
      *(ex+j) /= i;
      *(ex2+j) /= i;
      *(ex2+j) -= *(ex+j)**(ex+j);
      *(ex2+j) = sqrt(*(ex2+j));
    }

  p->PreProcData = (void*)ex;
}

static void OsivSetupPreProc_Method5 (osiv_param* p)
{
  osiv_intm i;
  osiv_intf j;
  osiv_imcp* im;
  osiv_real* avg;

  im = (osiv_imcp*)OsivMalloc(p->ImageLength*sizeof(osiv_imcp));
  avg = (osiv_real*)OsivCalloc(p->ImageLength,sizeof(osiv_real));

  for (i = p->FrmStart; i <= p->FrmFinish; i += p->FrmSkip)
    {
      OsivGetImage(p,i,im);
      for (j = 0; j < p->ImageLength; j++)
	{
	  *(avg+j) += *(im+j);
	}
    }

  i = (p->FrmFinish-p->FrmStart)/p->FrmSkip+1;
  for (j = 0; j < p->ImageLength; j++)
    {
      *(avg+j) /= i;
    }

  p->PreProcData = (void*)avg;
}

void OsivSetupPreProc (osiv_param* p)
{
  switch(p->PreProc)
    {
    case 0: break;
    case 1: OsivSetupPreProc_Method1(p); break;
    case 2: OsivSetupPreProc_Method2(p); break;
    case 3: OsivSetupPreProc_Method3(p); break;
    case 4: OsivSetupPreProc_Method4(p); break;
    case 5: OsivSetupPreProc_Method5(p); break;
    default:OsivError("unrecognized preprocessing method"); break;
    }
}

static void OsivPreProc_Method1 (osiv_param* p,
				 osiv_imcp* im)
{
  osiv_intf i;
  osiv_imcp* min;

  min = (osiv_imcp*)p->PreProcData;
  for (i = 0; i < p->ImageLength; i++)
    {
      if (*(im+i) < *(min+i)) *(im+i) = 0;
      else *(im+i) -= *(min+i);
    }
}

static void OsivPreProc_Method2 (osiv_param* p,
				 osiv_imcp* im)
{
  osiv_intf i;
  osiv_imcp* max;

  max = (osiv_imcp*)p->PreProcData;
  for (i = 0; i < p->ImageLength; i++)
    {
      if (*(im+i) > *(max+i)) *(im+i) = OSIV_IMCP_MAX;
      else *(im+i) += OSIV_IMCP_MAX - *(max+i);
    }
}

static void OsivPreProc_Method3 (osiv_param* p,
				 osiv_imcp* im)
{
  osiv_intf i;
  osiv_imcp* min;
  osiv_imcp* max;

  min = (osiv_imcp*)p->PreProcData;
  max = min+p->ImageLength;
  for (i = 0; i < p->ImageLength; i++)
    {
      register osiv_real c = *(im+i);

      c -= *(min+i);
      c *= OSIV_IMCP_MAX;
      c /= (osiv_real)*(max+i)-*(min+i);
      if (c < 0)
	*(im+i) = 0;
      else if (c > OSIV_IMCP_MAX)
	*(im+i) = OSIV_IMCP_MAX;
      else
	*(im+i) = c;
    }
}

static void OsivPreProc_Method4 (osiv_param* p,
				 osiv_imcp* im)
{
  osiv_intf i;
  osiv_real* avg;
  osiv_real* std;

  avg = (osiv_real*)p->PreProcData;
  std = avg+p->ImageLength;
  for (i = 0; i < p->ImageLength; i++)
    {
      register osiv_real c = *(im+i);

      c -= *(avg+i) - 2**(std+i);
      c *= OSIV_IMCP_MAX/(4**(std+i));
      if (c < 0)
	*(im+i) = 0;
      else if (c > OSIV_IMCP_MAX)
	*(im+i) = OSIV_IMCP_MAX;
      else
	*(im+i) = c;
    }
}

static void OsivPreProc_Method5 (osiv_param* p,
				 osiv_imcp* im)
{
  osiv_intf i;
  osiv_real* avg;

  avg = (osiv_real*)p->PreProcData;
  for (i = 0; i < p->ImageLength; i++)
    {
      register osiv_real c = *(im+i);

      c *= OSIV_IMCP_MAX/(2**(avg+i));
      *(im+i) = c<OSIV_IMCP_MAX ? c:OSIV_IMCP_MAX;
    }
}


void OsivPreProc (osiv_param* p,
		  osiv_intm frm,
		  osiv_imcp* image)
{
  switch(p->PreProc)
    {
    case 0: break;
    case 1: OsivPreProc_Method1(p,image); break;
    case 2: OsivPreProc_Method2(p,image); break;
    case 3: OsivPreProc_Method3(p,image); break;
    case 4: OsivPreProc_Method4(p,image); break;
    case 5: OsivPreProc_Method5(p,image); break;
    default: OsivError("unrecognized preprocessing method"); break;
    }
}

void OsivClosePreProc (osiv_param* p)
{
  switch (p->PreProc)
    {
    case 0: break;
    case 1: free(p->PreProcData); break;
    case 2: free(p->PreProcData); break;
    case 3: free(p->PreProcData); break;
    case 4: free(p->PreProcData); break;
    case 5: free(p->PreProcData); break;
    default: OsivError("unrecognized preprocessing method"); break;
    }
}
